@@ -1,5 +1,6 @@
 import json
 import logging
+import os
 import time
 from concurrent.futures import Future, ThreadPoolExecutor
 from pathlib import Path
@@ -15,7 +16,10 @@ from azure.mgmt.network import NetworkManagementClient
 from azure.mgmt.resource import ResourceManagementClient
 from azure.mgmt.resource.resources.models import DeploymentMode
 
+from ray._common.usage.usage_lib import get_cloud_from_metadata_requests
 from ray.autoscaler._private._azure.config import (
+    _delete_role_assignments_for_principal,
+    _generate_arm_guid,
     bootstrap_azure,
     get_azure_sdk_function,
 )
@@ -78,7 +82,30 @@ class AzureNodeProvider(NodeProvider):
                 f"to subscription_id from the logged in azure profile: {subscription_id}"
             )
         self.cache_stopped_nodes = provider_config.get("cache_stopped_nodes", True)
-        credential = DefaultAzureCredential(exclude_shared_token_cache_credential=True)
+
+        # Detect cloud environment to optimize Azure credential chain.
+        # On non-Azure clouds (AWS, GCP), skip Azure-specific auth methods
+        # (managed identity, workload identity) to avoid IMDS timeout delays / failures.
+        detected_cloud = get_cloud_from_metadata_requests()
+        on_azure = detected_cloud == "azure"
+
+        if on_azure:
+            logger.info(
+                "Initializing Azure node provider for Azure infrastructure "
+                "running on Azure cloud environment"
+            )
+        else:
+            logger.info(
+                f"Initializing Azure node provider for Azure infrastructure "
+                f"but detected this is running on a '{detected_cloud}' environment. "
+                f"Skipping Azure-specific authentication methods to avoid timeouts."
+            )
+
+        credential = DefaultAzureCredential(
+            exclude_shared_token_cache_credential=True,
+            exclude_managed_identity_credential=not on_azure,
+            exclude_workload_identity_credential=not on_azure,
+        )
         self.compute_client = ComputeManagementClient(credential, subscription_id)
         self.network_client = NetworkManagementClient(credential, subscription_id)
         self.resource_client = ResourceManagementClient(credential, subscription_id)
@@ -642,6 +669,267 @@ class AzureNodeProvider(NodeProvider):
         ):
             time.sleep(0.1)
 
+    def cleanup_cluster_resources(self):
+        """Delete shared cluster infrastructure (MSI, NSG, Subnet, VNet)."""
+
+        resource_group = self.provider_config["resource_group"]
+
+        msi_principal_id = self._cleanup_managed_identity(
+            resource_group, self.provider_config.get("msi")
+        )
+
+        subnet_id = self.provider_config.get("subnet")
+        vnet_name = self._cleanup_subnet(resource_group, subnet_id)
+
+        nsg_id = self.provider_config.get("nsg")
+        self._cleanup_nsg(resource_group, nsg_id)
+
+        self._cleanup_vnet(resource_group, subnet_id, vnet_name)
+
+        self._cleanup_role_assignments(resource_group, msi_principal_id)
+
+        self._prune_provider_config_entries()
+        self._cleanup_config_cache()
+
+    @staticmethod
+    def _get_resource_name_from_id(resource_id: Optional[str]) -> Optional[str]:
+        if resource_id:
+            return resource_id.split("/")[-1]
+        return None
+
+    @staticmethod
+    def _retry_delete(delete_fn, max_retries: int = 5, initial_delay: int = 2):
+        """Retry a delete operation with exponential backoff."""
+
+        delay = initial_delay
+        for attempt in range(max_retries):
+            try:
+                return delete_fn()
+            except Exception as exc:  # noqa: BLE001
+                error_msg = str(exc)
+                if "InUse" in error_msg and attempt < max_retries - 1:
+                    logger.info(
+                        "Resource still in use, retrying in %ss (attempt %s/%s)...",
+                        delay,
+                        attempt + 1,
+                        max_retries,
+                    )
+                    time.sleep(delay)
+                    delay *= 2
+                else:
+                    raise
+
+    def _cleanup_managed_identity(
+        self, resource_group: str, msi_id: Optional[str]
+    ) -> Optional[str]:
+        if not msi_id:
+            return None
+
+        msi_name = self._get_resource_name_from_id(msi_id)
+        if not msi_name:
+            return None
+
+        msi_principal_id: Optional[str] = None
+        try:
+            get_identity = get_azure_sdk_function(
+                client=self.resource_client.resources,
+                function_name="get_by_id",
+            )
+            existing_msi = get_identity(msi_id, "2023-01-31")
+            msi_principal_id = getattr(existing_msi, "properties", {}).get(
+                "principalId"
+            )
+        except ResourceNotFoundError:
+            msi_principal_id = None
+        except Exception as exc:  # noqa: BLE001
+            logger.warning(
+                "Failed to query MSI %s for principal ID prior to deletion: %s",
+                msi_name,
+                exc,
+            )
+
+        try:
+            logger.info("Deleting Managed Service Identity: %s", msi_name)
+            delete = get_azure_sdk_function(
+                client=self.resource_client.resources,
+                function_name="delete_by_id",
+            )
+            delete(resource_id=msi_id, api_version="2023-01-31").wait()
+            logger.info("Successfully deleted MSI: %s", msi_name)
+        except ResourceNotFoundError:
+            logger.info("MSI %s not found, may have been already deleted", msi_name)
+        except Exception as exc:  # noqa: BLE001
+            logger.warning("Failed to delete MSI %s: %s", msi_name, exc)
+
+        return msi_principal_id
+
+    def _cleanup_subnet(
+        self, resource_group: str, subnet_id: Optional[str]
+    ) -> Optional[str]:
+        if not subnet_id:
+            return None
+
+        subnet_name = self._get_resource_name_from_id(subnet_id)
+        vnet_name: Optional[str] = None
+
+        if subnet_id and "/virtualNetworks/" in subnet_id:
+            parts = subnet_id.split("/")
+            vnet_idx = parts.index("virtualNetworks")
+            if vnet_idx + 1 < len(parts):
+                vnet_name = parts[vnet_idx + 1]
+
+        if not subnet_name or not vnet_name:
+            return None
+
+        try:
+            logger.info("Deleting Subnet: %s in VNet: %s", subnet_name, vnet_name)
+
+            def delete_subnet():
+                delete = get_azure_sdk_function(
+                    client=self.network_client.subnets, function_name="delete"
+                )
+                delete(
+                    resource_group_name=resource_group,
+                    virtual_network_name=vnet_name,
+                    subnet_name=subnet_name,
+                ).wait()
+
+            self._retry_delete(delete_subnet)
+            logger.info("Successfully deleted Subnet: %s", subnet_name)
+        except ResourceNotFoundError:
+            logger.info(
+                "Subnet %s not found, may have been already deleted", subnet_name
+            )
+        except Exception as exc:  # noqa: BLE001
+            logger.warning("Failed to delete Subnet %s: %s", subnet_name, exc)
+
+        return vnet_name
+
+    def _cleanup_nsg(self, resource_group: str, nsg_id: Optional[str]) -> None:
+        if not nsg_id:
+            return
+
+        nsg_name = self._get_resource_name_from_id(nsg_id)
+        if not nsg_name:
+            return
+
+        try:
+            logger.info("Deleting Network Security Group: %s", nsg_name)
+
+            def delete_nsg():
+                delete = get_azure_sdk_function(
+                    client=self.network_client.network_security_groups,
+                    function_name="delete",
+                )
+                delete(
+                    resource_group_name=resource_group,
+                    network_security_group_name=nsg_name,
+                ).wait()
+
+            self._retry_delete(delete_nsg)
+            logger.info("Successfully deleted NSG: %s", nsg_name)
+        except ResourceNotFoundError:
+            logger.info("NSG %s not found, may have been already deleted", nsg_name)
+        except Exception as exc:  # noqa: BLE001
+            logger.warning("Failed to delete NSG %s: %s", nsg_name, exc)
+
+    def _cleanup_vnet(
+        self,
+        resource_group: str,
+        subnet_id: Optional[str],
+        vnet_name: Optional[str],
+    ) -> None:
+        if not subnet_id or not vnet_name:
+            return
+
+        try:
+            logger.info("Deleting Virtual Network: %s", vnet_name)
+
+            def delete_vnet():
+                delete = get_azure_sdk_function(
+                    client=self.network_client.virtual_networks,
+                    function_name="delete",
+                )
+                delete(
+                    resource_group_name=resource_group,
+                    virtual_network_name=vnet_name,
+                ).wait()
+
+            self._retry_delete(delete_vnet)
+            logger.info("Successfully deleted VNet: %s", vnet_name)
+        except ResourceNotFoundError:
+            logger.info("VNet %s not found, may have been already deleted", vnet_name)
+        except Exception as exc:  # noqa: BLE001
+            logger.warning("Failed to delete VNet %s: %s", vnet_name, exc)
+
+    def _cleanup_role_assignments(
+        self, resource_group: str, msi_principal_id: Optional[str]
+    ) -> None:
+        subscription_id = self.provider_config.get("subscription_id")
+        unique_id = self.provider_config.get("unique_id")
+        if not subscription_id or not unique_id:
+            return
+
+        cluster_id = f"{self.cluster_name}-{unique_id}"
+        role_assignment_name = f"ray-{cluster_id}-ra"
+        role_assignment_guid = _generate_arm_guid(role_assignment_name)
+        role_assignment_id = (
+            f"/subscriptions/{subscription_id}/resourceGroups/{resource_group}/providers"
+            f"/Microsoft.Authorization/roleAssignments/{role_assignment_guid}"
+        )
+
+        if msi_principal_id:
+            _delete_role_assignments_for_principal(
+                self.resource_client, resource_group, msi_principal_id
+            )
+
+        delete_role_assignment = get_azure_sdk_function(
+            client=self.resource_client.resources, function_name="delete_by_id"
+        )
+        try:
+            delete_lro = delete_role_assignment(
+                resource_id=role_assignment_id,
+                api_version="2022-04-01",
+            )
+            if hasattr(delete_lro, "wait"):
+                delete_lro.wait()
+            logger.info(
+                "Deleted role assignment %s for cluster %s",
+                role_assignment_guid,
+                self.cluster_name,
+            )
+        except ResourceNotFoundError:
+            logger.debug(
+                "Role assignment %s not found during cleanup", role_assignment_guid
+            )
+        except Exception as exc:  # noqa: BLE001
+            logger.warning(
+                "Failed to delete role assignment %s: %s",
+                role_assignment_guid,
+                exc,
+            )
+
+    def _prune_provider_config_entries(self) -> None:
+        for key in ("msi", "nsg", "subnet"):
+            self.provider_config.pop(key, None)
+
+    def _cleanup_config_cache(self) -> None:
+        cache_path = self.provider_config.get("_config_cache_path")
+        if not cache_path:
+            return
+
+        try:
+            if os.path.exists(cache_path):
+                os.remove(cache_path)
+                logger.info(
+                    "Deleted cached Ray config at %s after resource cleanup",
+                    cache_path,
+                )
+        except Exception as exc:  # noqa: BLE001
+            logger.warning("Failed to delete cached Ray config %s: %s", cache_path, exc)
+        finally:
+            self.provider_config.pop("_config_cache_path", None)
+
     def _get_node(self, node_id):
         self._get_filtered_nodes({})  # Side effect: updates cache
         return self.cached_nodes[node_id]
