@@ -14,6 +14,7 @@ import ray.dashboard.utils as dashboard_utils
 from ray import ActorID, NodeID
 from ray._common.network_utils import build_address
 from ray._common.usage.usage_constants import CLUSTER_METADATA_KEY
+from ray._private.grpc_utils import init_grpc_channel
 from ray._private.metrics_agent import PrometheusServiceDiscoveryWriter
 from ray._private.ray_constants import (
     DEBUG_AUTOSCALING_ERROR,
@@ -24,7 +25,6 @@ from ray._private.ray_constants import (
     KV_NAMESPACE_DASHBOARD,
     env_integer,
 )
-from ray._private.utils import init_grpc_channel
 from ray.autoscaler._private.commands import debug_status
 from ray.core.generated import reporter_pb2, reporter_pb2_grpc
 from ray.dashboard.consts import GCS_RPC_TIMEOUT_SECONDS
