@@ -1,8 +1,6 @@
-import copy
 import logging
 import math
 import os
-import warnings
 from dataclasses import dataclass
 from typing import (
     TYPE_CHECKING,
@@ -23,7 +21,10 @@ from packaging.version import parse as parse_version
 
 import ray
 from ray._private.arrow_utils import get_pyarrow_version
-from ray.data._internal.arrow_block import ArrowBlockAccessor
+from ray.data._internal.arrow_block import (
+    _BATCH_SIZE_PRESERVING_STUB_COL_NAME,
+    ArrowBlockAccessor,
+)
 from ray.data._internal.progress_bar import ProgressBar
 from ray.data._internal.remote_fn import cached_remote_fn
 from ray.data._internal.util import (
@@ -49,7 +50,6 @@ from ray.data.datasource.partitioning import (
     PathPartitionParser,
 )
 from ray.data.datasource.path_util import (
-    _has_file_extension,
     _resolve_paths_and_filesystem,
 )
 from ray.util.debug import log_once
@@ -104,9 +104,6 @@ PARQUET_ENCODING_RATIO_ESTIMATE_MAX_NUM_SAMPLES = 10
 PARQUET_ENCODING_RATIO_ESTIMATE_NUM_ROWS = 1024
 
 
-_BATCH_SIZE_PRESERVING_STUB_COL_NAME = "__bsp_stub"
-
-
 class _ParquetFragment:
     """This wrapper class is created to avoid utilizing `ParquetFileFragment` original
     serialization protocol that actually does network RPCs during serialization
@@ -172,7 +169,7 @@ class ParquetDatasource(Datasource):
     cost of some potential performance and/or compatibility penalties.
     """
 
-    _FUTURE_FILE_EXTENSIONS = ["parquet"]
+    _FILE_EXTENSIONS = ["parquet"]
 
     def __init__(
         self,
@@ -191,6 +188,7 @@ class ParquetDatasource(Datasource):
         include_paths: bool = False,
         file_extensions: Optional[List[str]] = None,
     ):
+        super().__init__()
         _check_pyarrow_version()
 
         self._supports_distributed_reads = not _is_local_scheme(paths)
@@ -208,7 +206,8 @@ class ParquetDatasource(Datasource):
             self._local_scheduling = NodeAffinitySchedulingStrategy(
                 ray.get_runtime_context().get_node_id(), soft=False
             )
-
+        # Need this property for lineage tracking
+        self._source_paths = paths
         paths, self._filesystem = _resolve_paths_and_filesystem(paths, filesystem)
         filesystem = RetryingPyFileSystem.wrap(
             self._filesystem,
@@ -273,7 +272,12 @@ class ParquetDatasource(Datasource):
         self._pq_paths = [p.path for p in pq_ds.fragments]
         self._block_udf = _block_udf
         self._to_batches_kwargs = to_batch_kwargs
-        self._data_columns = data_columns
+        # Store as projection_map (identity mapping if columns specified, None otherwise)
+        # Note: Empty list [] means no columns, None means all columns
+        if data_columns is None:
+            self._projection_map = None
+        else:
+            self._projection_map = {col: col for col in data_columns}
         self._partition_columns = partition_columns
         self._read_schema = schema
         self._file_schema = pq_ds.schema
@@ -283,7 +287,6 @@ class ParquetDatasource(Datasource):
         self._file_metadata_shuffler = None
         self._include_paths = include_paths
         self._partitioning = partitioning
-
         if shuffle == "files":
             self._file_metadata_shuffler = np.random.default_rng()
         elif isinstance(shuffle, FileShuffleConfig):
@@ -300,7 +303,7 @@ class ParquetDatasource(Datasource):
 
         sampled_file_infos = _fetch_file_infos(
             sampled_fragments,
-            columns=self._data_columns,
+            columns=self._get_data_columns(),
             schema=schema,
             local_scheduling=self._local_scheduling,
         )
@@ -314,17 +317,9 @@ class ParquetDatasource(Datasource):
             sampled_file_infos, DataContext.get_current().target_max_block_size
         )
 
-        if file_extensions is None:
-            for path in self._pq_paths:
-                if not _has_file_extension(
-                    path, self._FUTURE_FILE_EXTENSIONS
-                ) and log_once("read_parquet_file_extensions_future_warning"):
-                    emit_file_extensions_future_warning(self._FUTURE_FILE_EXTENSIONS)
-                    break
-
     def estimate_inmemory_data_size(self) -> int:
         # In case of empty projections no data will be read
-        if self._data_columns == []:
+        if self._projection_map == {}:
             return 0
 
         return self._estimate_in_mem_size(self._pq_fragments)
@@ -359,6 +354,12 @@ class ParquetDatasource(Datasource):
         )
 
         read_tasks = []
+        filter_expr = (
+            self._predicate_expr.to_pyarrow()
+            if self._predicate_expr is not None
+            else None
+        )
+
         for fragments, paths in zip(
             np.array_split(pq_fragments, parallelism),
             np.array_split(pq_paths, parallelism),
@@ -378,6 +379,7 @@ class ParquetDatasource(Datasource):
                 to_batches_kwargs,
                 default_read_batch_size_rows,
                 data_columns,
+                data_columns_rename_map,
                 partition_columns,
                 read_schema,
                 include_paths,
@@ -386,7 +388,8 @@ class ParquetDatasource(Datasource):
                 self._block_udf,
                 self._to_batches_kwargs,
                 self._default_batch_size,
-                self._data_columns,
+                self._get_data_columns(),
+                self.get_column_renames(),
                 self._partition_columns,
                 self._read_schema,
                 self._include_paths,
@@ -400,11 +403,13 @@ class ParquetDatasource(Datasource):
                         to_batches_kwargs,
                         default_read_batch_size_rows,
                         data_columns,
+                        data_columns_rename_map,
                         partition_columns,
                         read_schema,
                         f,
                         include_paths,
                         partitioning,
+                        filter_expr,
                     ),
                     meta,
                     schema=target_schema,
@@ -428,18 +433,18 @@ class ParquetDatasource(Datasource):
     def supports_projection_pushdown(self) -> bool:
         return True
 
+    def supports_predicate_pushdown(self) -> bool:
+        return True
+
     def get_current_projection(self) -> Optional[List[str]]:
+        """Override to include partition columns in addition to data columns."""
         # NOTE: In case there's no projection both file and partition columns
         #       will be none
-        if self._data_columns is None and self._partition_columns is None:
+        data_columns = self._get_data_columns()
+        if data_columns is None and self._partition_columns is None:
             return None
 
-        return (self._data_columns or []) + (self._partition_columns or [])
-
-    def apply_projection(self, columns: List[str]) -> "ParquetDatasource":
-        clone = copy.copy(self)
-        clone._data_columns = columns
-        return clone
+        return (data_columns or []) + (self._partition_columns or [])
 
     def _estimate_in_mem_size(self, fragments: List[_ParquetFragment]) -> int:
         in_mem_size = sum([f.file_size for f in fragments]) * self._encoding_ratio
@@ -452,11 +457,13 @@ def read_fragments(
     to_batches_kwargs: Dict[str, Any],
     default_read_batch_size_rows: Optional[int],
     data_columns: Optional[List[str]],
+    data_columns_rename_map: Optional[Dict[str, str]],
     partition_columns: Optional[List[str]],
     schema: Optional[Union[type, "pyarrow.lib.Schema"]],
     fragments: List[_ParquetFragment],
     include_paths: bool,
     partitioning: Partitioning,
+    filter_expr: Optional["pyarrow.dataset.Expression"] = None,
 ) -> Iterator["pyarrow.Table"]:
     # This import is necessary to load the tensor extension type.
     from ray.data.extensions.tensor_extension import ArrowTensorType  # noqa
@@ -474,9 +481,11 @@ def read_fragments(
                 fragment.original,
                 schema=schema,
                 data_columns=data_columns,
+                data_columns_rename_map=data_columns_rename_map,
                 partition_columns=partition_columns,
                 partitioning=partitioning,
                 include_path=include_paths,
+                filter_expr=filter_expr,
                 batch_size=default_read_batch_size_rows,
                 to_batches_kwargs=to_batches_kwargs,
             ),
@@ -496,6 +505,7 @@ def _read_batches_from(
     *,
     schema: "pyarrow.Schema",
     data_columns: Optional[List[str]],
+    data_columns_rename_map: Optional[Dict[str, str]],
     partition_columns: Optional[List[str]],
     partitioning: Partitioning,
     filter_expr: Optional["pyarrow.dataset.Expression"] = None,
@@ -508,13 +518,22 @@ def _read_batches_from(
 
     import pyarrow as pa
 
+    from ray.data.datasource.datasource import _DatasourceProjectionPushdownMixin
+
     # Copy to avoid modifying passed in arg
     to_batches_kwargs = dict(to_batches_kwargs or {})
 
     # NOTE: Passed in kwargs overrides always take precedence
     # TODO deprecate to_batches_kwargs
     use_threads = to_batches_kwargs.pop("use_threads", use_threads)
-    filter_expr = to_batches_kwargs.pop("filter", filter_expr)
+    # TODO: We should deprecate filter through the read_parquet API and only allow through dataset.filter()
+    filter_from_kwargs = to_batches_kwargs.pop("filter", None)
+    if filter_from_kwargs is not None:
+        filter_expr = (
+            filter_from_kwargs
+            if filter_expr is None
+            else filter_expr & filter_from_kwargs
+        )
     # NOTE: Arrow's ``to_batches`` expects ``batch_size`` as an int
     if batch_size is not None:
         to_batches_kwargs.setdefault("batch_size", batch_size)
@@ -523,56 +542,66 @@ def _read_batches_from(
         fragment, partition_columns, partitioning
     )
 
-    try:
-        for batch in fragment.to_batches(
-            columns=data_columns,
-            filter=filter_expr,
-            schema=schema,
-            use_threads=use_threads,
-            **to_batches_kwargs,
-        ):
-            table = pa.Table.from_batches([batch])
+    def _generate_tables() -> "pa.Table":
+        """Inner generator that yields tables without renaming."""
+        try:
+            for batch in fragment.to_batches(
+                columns=data_columns,
+                filter=filter_expr,
+                schema=schema,
+                use_threads=use_threads,
+                **to_batches_kwargs,
+            ):
+                table = pa.Table.from_batches([batch])
 
-            if include_path:
-                table = ArrowBlockAccessor.for_block(table).fill_column(
-                    "path", fragment.path
+                if include_path:
+                    table = ArrowBlockAccessor.for_block(table).fill_column(
+                        "path", fragment.path
+                    )
+
+                if partition_col_values:
+                    table = _add_partitions_to_table(partition_col_values, table)
+
+                # ``ParquetFileFragment.to_batches`` returns ``RecordBatch``,
+                # which could have empty projection (ie ``num_columns`` == 0)
+                # while having non-empty rows (ie ``num_rows`` > 0), which
+                # could occur when list of requested columns is empty.
+                #
+                # However, when ``RecordBatches`` are concatenated using
+                # ``pyarrow.concat_tables`` it will return a single ``Table``
+                # with 0 columns and therefore 0 rows (since ``Table``s number of
+                # rows is determined as the length of its columns).
+                #
+                # To avoid running into this pitfall, we introduce a stub column
+                # holding just nulls to maintain invariance of the number of rows.
+                #
+                # NOTE: There's no impact from this as the binary size of the
+                #       extra column is basically 0
+                if table.num_columns == 0 and table.num_rows > 0:
+                    table = table.append_column(
+                        _BATCH_SIZE_PRESERVING_STUB_COL_NAME, pa.nulls(table.num_rows)
+                    )
+
+                yield table
+
+        except pa.lib.ArrowInvalid as e:
+            error_message = str(e)
+            if (
+                "No match for FieldRef.Name" in error_message
+                and filter_expr is not None
+            ):
+                filename = os.path.basename(fragment.path)
+                file_columns = set(fragment.physical_schema.names)
+                raise RuntimeError(
+                    f"Filter expression: '{filter_expr}' failed on parquet "
+                    f"file: '{filename}' with columns: {file_columns}"
                 )
+            raise
 
-            if partition_col_values:
-                table = _add_partitions_to_table(partition_col_values, table)
-
-            # ``ParquetFileFragment.to_batches`` returns ``RecordBatch``,
-            # which could have empty projection (ie ``num_columns`` == 0)
-            # while having non-empty rows (ie ``num_rows`` > 0), which
-            # could occur when list of requested columns is empty.
-            #
-            # However, when ``RecordBatches`` are concatenated using
-            # ``pyarrow.concat_tables`` it will return a single ``Table``
-            # with 0 columns and therefore 0 rows (since ``Table``s number of
-            # rows is determined as the length of its columns).
-            #
-            # To avoid running into this pitfall, we introduce a stub column
-            # holding just nulls to maintain invariance of the number of rows.
-            #
-            # NOTE: There's no impact from this as the binary size of the
-            #       extra column is basically 0
-            if table.num_columns == 0 and table.num_rows > 0:
-                table = table.append_column(
-                    _BATCH_SIZE_PRESERVING_STUB_COL_NAME, pa.nulls(table.num_rows)
-                )
-
-            yield table
-
-    except pa.lib.ArrowInvalid as e:
-        error_message = str(e)
-        if "No match for FieldRef.Name" in error_message and filter_expr is not None:
-            filename = os.path.basename(fragment.path)
-            file_columns = set(fragment.physical_schema.names)
-            raise RuntimeError(
-                f"Filter expression: '{filter_expr}' failed on parquet "
-                f"file: '{filename}' with columns: {file_columns}"
-            )
-        raise
+    # Apply renames to all tables from the generator
+    yield from _DatasourceProjectionPushdownMixin._apply_rename_to_tables(
+        _generate_tables(), data_columns_rename_map
+    )
 
 
 def _parse_partition_column_values(
@@ -860,16 +889,6 @@ def _get_partition_columns_schema(
     return pa.schema(fields)
 
 
-def emit_file_extensions_future_warning(future_file_extensions: List[str]):
-    warnings.warn(
-        "The default `file_extensions` for `read_parquet` will change "
-        f"from `None` to {future_file_extensions} after Ray 2.43, and your dataset "
-        "contains files that don't match the new `file_extensions`. To maintain "
-        "backwards compatibility, set `file_extensions=None` explicitly.",
-        FutureWarning,
-    )
-
-
 def _derive_schema(
     read_schema: Optional["pyarrow.Schema"],
     *,
