@@ -20,6 +20,7 @@ from ray.data._internal.compute import ActorPoolStrategy
 from ray.data._internal.execution.bundle_queue import create_bundle_queue
 from ray.data._internal.execution.bundle_queue.bundle_queue import BundleQueue
 from ray.data._internal.execution.interfaces import (
+    BlockSlice,
     ExecutionOptions,
     ExecutionResources,
     NodeIdStr,
@@ -32,12 +33,19 @@ from ray.data._internal.execution.node_trackers.actor_location import (
     ActorLocationTracker,
     get_or_create_actor_location_tracker,
 )
-from ray.data._internal.execution.operators.map_operator import MapOperator, _map_task
+from ray.data._internal.execution.operators.map_operator import (
+    BaseRefBundler,
+    MapOperator,
+    _map_task,
+)
 from ray.data._internal.execution.operators.map_transformer import MapTransformer
 from ray.data._internal.execution.util import locality_string
 from ray.data._internal.remote_fn import _add_system_error_to_retry_exceptions
 from ray.data.block import Block, BlockMetadata
-from ray.data.context import DataContext
+from ray.data.context import (
+    DEFAULT_ACTOR_MAX_TASKS_IN_FLIGHT_TO_MAX_CONCURRENCY_FACTOR,
+    DataContext,
+)
 from ray.types import ObjectRef
 from ray.util.common import INT32_MAX
 
@@ -68,10 +76,12 @@ class ActorPoolMapOperator(MapOperator):
         compute_strategy: ActorPoolStrategy,
         name: str = "ActorPoolMap",
         min_rows_per_bundle: Optional[int] = None,
+        ref_bundler: Optional[BaseRefBundler] = None,
         supports_fusion: bool = True,
         map_task_kwargs: Optional[Dict[str, Any]] = None,
         ray_remote_args_fn: Optional[Callable[[], Dict[str, Any]]] = None,
         ray_remote_args: Optional[Dict[str, Any]] = None,
+        ray_actor_task_remote_args: Optional[Dict[str, Any]] = None,
         target_max_block_size_override: Optional[int] = None,
     ):
         """Create an ActorPoolMapOperator instance.
@@ -87,6 +97,7 @@ class ActorPoolMapOperator(MapOperator):
                 transform_fn, or None to use the block size. Setting the batch size is
                 important for the performance of GPU-accelerated transform functions.
                 The actual rows passed may be less if the dataset is small.
+            ref_bundler: The ref bundler to use for this operator.
             supports_fusion: Whether this operator supports fusion with other operators.
             map_task_kwargs: A dictionary of kwargs to pass to the map task. You can
                 access these kwargs through the `TaskContext.kwargs` dictionary.
@@ -98,6 +109,7 @@ class ActorPoolMapOperator(MapOperator):
                 advanced, experimental feature.
             ray_remote_args: Customize the ray remote args for this op's tasks.
                 See :func:`ray.remote` for details.
+            ray_actor_task_remote_args: Ray Core options passed to map actor tasks.
             target_max_block_size_override: The target maximum number of bytes to
                 include in an output block.
         """
@@ -108,34 +120,21 @@ class ActorPoolMapOperator(MapOperator):
             name,
             target_max_block_size_override,
             min_rows_per_bundle,
+            ref_bundler,
             supports_fusion,
             map_task_kwargs,
             ray_remote_args_fn,
             ray_remote_args,
         )
-        self._ray_actor_task_remote_args = {}
-        actor_task_errors = self.data_context.actor_task_retry_on_errors
-        if actor_task_errors:
-            self._ray_actor_task_remote_args["retry_exceptions"] = actor_task_errors
-        _add_system_error_to_retry_exceptions(self._ray_actor_task_remote_args)
-
-        if (
-            "_generator_backpressure_num_objects"
-            not in self._ray_actor_task_remote_args
-            and self.data_context._max_num_blocks_in_streaming_gen_buffer is not None
-        ):
-            # The `_generator_backpressure_num_objects` parameter should be
-            # `2 * _max_num_blocks_in_streaming_gen_buffer` because we yield
-            # 2 objects for each block: the block and the block metadata.
-            self._ray_actor_task_remote_args["_generator_backpressure_num_objects"] = (
-                2 * self.data_context._max_num_blocks_in_streaming_gen_buffer
-            )
 
         self._min_rows_per_bundle = min_rows_per_bundle
         self._ray_remote_args_fn = ray_remote_args_fn
         self._ray_remote_args = self._apply_default_remote_args(
             self._ray_remote_args, self.data_context
         )
+        self._ray_actor_task_remote_args = self._apply_default_actor_task_remote_args(
+            ray_actor_task_remote_args, self.data_context
+        )
 
         per_actor_resource_usage = ExecutionResources(
             cpu=self._ray_remote_args.get("num_cpus"),
@@ -153,12 +152,14 @@ class ActorPoolMapOperator(MapOperator):
             initial_size=compute_strategy.initial_size,
             max_actor_concurrency=max_actor_concurrency,
             max_tasks_in_flight_per_actor=(
-                # NOTE: Unless explicitly configured by the user, max tasks-in-flight config
-                #       will fall back to be 2 x of `max_concurrency`, entailing that for every
-                #       running task we'd allow 1 more task to be enqueued
+                # Unless explicitly overridden by the user, max tasks-in-flight config
+                # will fall back to be:
+                #
+                #   DEFAULT_ACTOR_MAX_TASKS_IN_FLIGHT_TO_MAX_CONCURRENCY_FACTOR * max_concurrency,
                 compute_strategy.max_tasks_in_flight_per_actor
                 or data_context.max_tasks_in_flight_per_actor
-                or max_actor_concurrency * 2
+                or max_actor_concurrency
+                * DEFAULT_ACTOR_MAX_TASKS_IN_FLIGHT_TO_MAX_CONCURRENCY_FACTOR
             ),
             _enable_actor_pool_on_exit_hook=self.data_context._enable_actor_pool_on_exit_hook,
         )
@@ -173,6 +174,7 @@ class ActorPoolMapOperator(MapOperator):
         self._actor_cls = None
         # Whether no more submittable bundles will be added.
         self._inputs_done = False
+        self._actor_locality_enabled: Optional[bool] = None
 
         # Locality metrics
         self._locality_hits = 0
@@ -182,18 +184,44 @@ class ActorPoolMapOperator(MapOperator):
     def _create_task_selector(actor_pool: "_ActorPool") -> "_ActorTaskSelector":
         return _ActorTaskSelectorImpl(actor_pool)
 
-    def internal_queue_size(self) -> int:
+    @staticmethod
+    def _apply_default_actor_task_remote_args(
+        ray_actor_task_remote_args: Optional[Dict[str, Any]], data_context: DataContext
+    ) -> Dict[str, Any]:
+        """Apply defaults to the actor task remote args."""
+        if ray_actor_task_remote_args is None:
+            ray_actor_task_remote_args = {}
+
+        ray_actor_task_remote_args = ray_actor_task_remote_args.copy()
+
+        actor_task_errors = data_context.actor_task_retry_on_errors
+        if actor_task_errors:
+            ray_actor_task_remote_args["retry_exceptions"] = actor_task_errors
+        _add_system_error_to_retry_exceptions(ray_actor_task_remote_args)
+
+        if (
+            "_generator_backpressure_num_objects" not in ray_actor_task_remote_args
+            and data_context._max_num_blocks_in_streaming_gen_buffer is not None
+        ):
+            # The `_generator_backpressure_num_objects` parameter should be
+            # `2 * _max_num_blocks_in_streaming_gen_buffer` because we yield
+            # 2 objects for each block: the block and the block metadata.
+            ray_actor_task_remote_args["_generator_backpressure_num_objects"] = (
+                2 * data_context._max_num_blocks_in_streaming_gen_buffer
+            )
+
+        return ray_actor_task_remote_args
+
+    def internal_input_queue_num_blocks(self) -> int:
         # NOTE: Internal queue size for ``ActorPoolMapOperator`` includes both
         #   - Input blocks bundler, alas
         #   - Own bundle's queue
-        return self._block_ref_bundler.num_bundles() + len(self._bundle_queue)
+        return self._block_ref_bundler.num_blocks() + self._bundle_queue.num_blocks()
 
-    def completed(self) -> bool:
-        # TODO separate marking as completed from the check
+    def internal_input_queue_num_bytes(self) -> int:
         return (
-            self._inputs_complete
-            and self._bundle_queue.is_empty()
-            and super().completed()
+            self._bundle_queue.estimate_size_bytes()
+            + self._block_ref_bundler.size_bytes()
         )
 
     def start(self, options: ExecutionOptions):
@@ -310,6 +338,7 @@ class ActorPoolMapOperator(MapOperator):
                 self.data_context,
                 ctx,
                 *input_blocks,
+                slices=bundle.slices,
                 **self.get_map_task_kwargs(),
             )
 
@@ -482,14 +511,13 @@ class ActorPoolMapOperator(MapOperator):
     def per_task_resource_allocation(
         self: "PhysicalOperator",
     ) -> ExecutionResources:
-        max_concurrency = self._ray_remote_args.get("max_concurrency", 1)
+        # For Actor tasks resource allocation is determined as:
+        #   - Per actor resource allocation divided by
+        #   - Actor's max task concurrency
+        max_concurrency = self._actor_pool.max_actor_concurrency()
         per_actor_resource_usage = self._actor_pool.per_actor_resource_usage()
         return per_actor_resource_usage.scale(1 / max_concurrency)
 
-    def max_task_concurrency(self: "PhysicalOperator") -> Optional[int]:
-        max_concurrency = self._ray_remote_args.get("max_concurrency", 1)
-        return max_concurrency * self._actor_pool.max_size()
-
     def min_scheduling_resources(
         self: "PhysicalOperator",
     ) -> ExecutionResources:
@@ -517,6 +545,9 @@ class ActorPoolMapOperator(MapOperator):
         """Returns Actor counts for Alive, Restarting and Pending Actors."""
         return self._actor_pool.get_actor_info()
 
+    def get_max_concurrency_limit(self) -> Optional[int]:
+        return self._actor_pool.max_size() * self._actor_pool.max_actor_concurrency()
+
 
 class _MapWorker:
     """An actor worker for MapOperator."""
@@ -527,7 +558,7 @@ class _MapWorker:
         src_fn_name: str,
         map_transformer: MapTransformer,
         logical_actor_id: str,
-        actor_location_tracker: ActorLocationTracker,
+        actor_location_tracker: ray.actor.ActorHandle[ActorLocationTracker],
     ):
         self.src_fn_name: str = src_fn_name
         self._map_transformer = map_transformer
@@ -549,6 +580,7 @@ class _MapWorker:
         data_context: DataContext,
         ctx: TaskContext,
         *blocks: Block,
+        slices: Optional[List[BlockSlice]] = None,
         **kwargs: Dict[str, Any],
     ) -> Iterator[Union[Block, List[BlockMetadata]]]:
         yield from _map_task(
@@ -556,6 +588,7 @@ class _MapWorker:
             data_context,
             ctx,
             *blocks,
+            slices=slices,
             **kwargs,
         )
 
