@@ -3,6 +3,7 @@ import functools
 import itertools
 import logging
 import math
+import random
 import threading
 import time
 from collections import defaultdict, deque
@@ -16,6 +17,7 @@ from typing import (
     Dict,
     List,
     Optional,
+    Set,
     Tuple,
     Union,
 )
@@ -25,14 +27,16 @@ import pyarrow as pa
 
 import ray
 from ray import ObjectRef
+from ray._private.ray_constants import env_integer
 from ray.actor import ActorHandle
-from ray.data import DataContext, ExecutionOptions, ExecutionResources
 from ray.data._internal.arrow_block import ArrowBlockBuilder
 from ray.data._internal.arrow_ops.transform_pyarrow import (
     _create_empty_table,
     hash_partition,
 )
 from ray.data._internal.execution.interfaces import (
+    ExecutionOptions,
+    ExecutionResources,
     PhysicalOperator,
     RefBundle,
 )
@@ -43,6 +47,8 @@ from ray.data._internal.execution.interfaces.physical_operator import (
     _create_sub_pb,
     estimate_total_num_of_blocks,
 )
+from ray.data._internal.execution.operators.sub_progress import SubProgressBarMixin
+from ray.data._internal.logical.interfaces import LogicalOperator
 from ray.data._internal.stats import OpRuntimeMetrics
 from ray.data._internal.table_block import TableBlockAccessor
 from ray.data._internal.util import GiB, MiB
@@ -56,6 +62,11 @@ from ray.data.block import (
     BlockType,
     to_stats,
 )
+from ray.data.context import (
+    DEFAULT_MAX_HASH_SHUFFLE_AGGREGATORS,
+    DEFAULT_TARGET_MAX_BLOCK_SIZE,
+    DataContext,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -67,6 +78,15 @@ StatefulShuffleAggregationFactory = Callable[
 ]
 
 
+DEFAULT_HASH_SHUFFLE_AGGREGATOR_MAX_CONCURRENCY = env_integer(
+    "RAY_DATA_DEFAULT_HASH_SHUFFLE_AGGREGATOR_MAX_CONCURRENCY", 8
+)
+
+DEFAULT_HASH_SHUFFLE_AGGREGATOR_MEMORY_ALLOCATION = env_integer(
+    "RAY_DATA_DEFAULT_HASH_SHUFFLE_AGGREGATOR_MEMORY_ALLOCATION", 1 * GiB
+)
+
+
 class StatefulShuffleAggregation(abc.ABC):
     """Interface for a stateful aggregation to be used by hash-based shuffling
     operators (inheriting from `HashShufflingOperatorBase`) and subsequent
@@ -308,7 +328,7 @@ def _shuffle_block(
 
         logger.debug(
             f"Shuffled block (rows={original_block_metadata.num_rows}, "
-            f"bytes={original_block_metadata.size_bytes/MiB:.2f}MB) "
+            f"bytes={original_block_metadata.size_bytes/MiB:.1f}MB) "
             f"into {len(partition_shards_stats)} partitions ("
             f"quantiles={'/'.join(map(str, quantiles))}, "
             f"rows={'/'.join(map(str, num_rows_quantiles))}, "
@@ -346,7 +366,7 @@ class _PartitionStats:
         )
 
 
-class HashShuffleProgressBarMixin(abc.ABC):
+class HashShuffleProgressBarMixin(SubProgressBarMixin):
     @property
     @abc.abstractmethod
     def shuffle_name(self) -> str:
@@ -357,26 +377,29 @@ class HashShuffleProgressBarMixin(abc.ABC):
     def reduce_name(self) -> str:
         ...
 
+    def _validate_sub_progress_bar_names(self):
+        assert self.shuffle_name is not None, "shuffle_name should not be None"
+        assert self.reduce_name is not None, "reduce_name should not be None"
+
     def initialize_sub_progress_bars(self, position: int) -> int:
-        """Display all sub progres bars in the termainl, and return the number of bars."""
+        """Display all sub progress bars in the termainl, and return the number of bars."""
+        self._validate_sub_progress_bar_names()
 
         # shuffle
         progress_bars_created = 0
         self.shuffle_bar = None
-        if self.shuffle_name is not None:
-            self.shuffle_bar, position = _create_sub_pb(
-                self.shuffle_name, self.num_output_rows_total(), position
-            )
-            progress_bars_created += 1
+        self.shuffle_bar, position = _create_sub_pb(
+            self.shuffle_name, self.num_output_rows_total(), position
+        )
+        progress_bars_created += 1
         self.shuffle_metrics = OpRuntimeMetrics(self)
 
         # reduce
         self.reduce_bar = None
-        if self.reduce_name is not None:
-            self.reduce_bar, position = _create_sub_pb(
-                self.reduce_name, self.num_output_rows_total(), position
-            )
-            progress_bars_created += 1
+        self.reduce_bar, position = _create_sub_pb(
+            self.reduce_name, self.num_output_rows_total(), position
+        )
+        progress_bars_created += 1
         self.reduce_metrics = OpRuntimeMetrics(self)
 
         return progress_bars_created
@@ -386,6 +409,54 @@ class HashShuffleProgressBarMixin(abc.ABC):
         self.shuffle_bar.close()
         self.reduce_bar.close()
 
+    def get_sub_progress_bar_names(self) -> Optional[List[str]]:
+        self._validate_sub_progress_bar_names()
+
+        # shuffle
+        self.shuffle_bar = None
+        self.shuffle_metrics = OpRuntimeMetrics(self)
+
+        # reduce
+        self.reduce_bar = None
+        self.reduce_metrics = OpRuntimeMetrics(self)
+
+        return [self.shuffle_name, self.reduce_name]
+
+    def set_sub_progress_bar(self, name, pg):
+        # No type-hints due to circular imports. `name` should be a `str`
+        # and `pg` should be a `SubProgressBar`
+        if self.shuffle_name is not None and self.shuffle_name == name:
+            self.shuffle_bar = pg
+        elif self.reduce_name is not None and self.reduce_name == name:
+            self.reduce_bar = pg
+
+
+def _derive_max_shuffle_aggregators(
+    total_cluster_resources: ExecutionResources,
+    data_context: DataContext,
+) -> int:
+    # Motivation for derivation of max # of shuffle aggregators is based on the
+    # following observations:
+    #
+    #   - Shuffle operation is necessarily a terminal operation: it terminates current
+    #     shuffle stage (set of operators that can execute concurrently)
+    #   - Shuffle operation has very low computation footprint until all preceding
+    #     operation completes (ie until shuffle finalization)
+    #   - When shuffle is finalized only shuffle operator is executing (ie it has
+    #     all of the cluster resources available at its disposal)
+    #
+    # As such we establish that the max number of shuffle
+    # aggregators (workers):
+    #
+    #   - Should not exceed total # of CPUs (to fully utilize cluster resources
+    #   while avoiding thrashing these due to over-allocation)
+    #   - Should be capped at fixed size (128 by default)
+    return min(
+        math.ceil(total_cluster_resources.cpu),
+        data_context.max_hash_shuffle_aggregators
+        or DEFAULT_MAX_HASH_SHUFFLE_AGGREGATORS,
+    )
+
 
 class HashShufflingOperatorBase(PhysicalOperator, HashShuffleProgressBarMixin):
     """Physical operator base-class for any operators requiring hash-based
@@ -409,26 +480,45 @@ class HashShufflingOperatorBase(PhysicalOperator, HashShuffleProgressBarMixin):
 
     def __init__(
         self,
-        name: str,
+        name_factory: Callable[[int], str],
         input_ops: List[PhysicalOperator],
         data_context: DataContext,
         *,
         key_columns: List[Tuple[str]],
-        num_partitions: int,
         partition_aggregation_factory: StatefulShuffleAggregationFactory,
+        num_partitions: Optional[int] = None,
         partition_size_hint: Optional[int] = None,
         input_block_transformer: Optional[BlockTransformer] = None,
         aggregator_ray_remote_args_override: Optional[Dict[str, Any]] = None,
         shuffle_progress_bar_name: Optional[str] = None,
         finalize_progress_bar_name: Optional[str] = None,
     ):
+        input_logical_ops = [
+            input_physical_op._logical_operators[0] for input_physical_op in input_ops
+        ]
+
+        estimated_input_blocks = [
+            input_op.estimated_num_outputs() for input_op in input_logical_ops
+        ]
+
+        # Derive target num partitions as either of
+        #   - Requested target number of partitions
+        #   - Max estimated target number of blocks generated by the input op(s)
+        #   - Default configured hash-shuffle parallelism (200)
+        target_num_partitions: int = (
+            num_partitions
+            or (max(estimated_input_blocks) if all(estimated_input_blocks) else None)
+            or data_context.default_hash_shuffle_parallelism
+        )
+
         super().__init__(
-            name=name,
+            name=name_factory(target_num_partitions),
             input_dependencies=input_ops,
             data_context=data_context,
-            target_max_block_size=None,
         )
 
+        assert partition_size_hint is None or partition_size_hint > 0
+
         if shuffle_progress_bar_name is None:
             shuffle_progress_bar_name = "Shuffle"
         if finalize_progress_bar_name is None:
@@ -443,29 +533,45 @@ class HashShufflingOperatorBase(PhysicalOperator, HashShuffleProgressBarMixin):
         )
 
         self._key_column_names: List[Tuple[str]] = key_columns
-        self._num_partitions = num_partitions
+        self._num_partitions: int = target_num_partitions
 
         # Determine max number of shuffle aggregators (defaults to
         # `DataContext.min_parallelism`)
-        max_shuffle_aggregators = (
-            data_context.max_hash_shuffle_aggregators
-            or data_context.default_hash_shuffle_parallelism
+        total_available_cluster_resources = _get_total_cluster_resources()
+        max_shuffle_aggregators = _derive_max_shuffle_aggregators(
+            total_available_cluster_resources, data_context
         )
+
         # Cap number of aggregators to not exceed max configured
-        num_aggregators = min(num_partitions, max_shuffle_aggregators)
+        num_aggregators = min(target_num_partitions, max_shuffle_aggregators)
+
+        # Target dataset's size estimated as either of
+        #   1. ``partition_size_hint`` multiplied by target number of partitions
+        #   2. Estimation of input ops' outputs bytes
+        if partition_size_hint is not None:
+            # TODO replace with dataset-byte-size hint
+            estimated_dataset_bytes = partition_size_hint * target_num_partitions
+        else:
+            estimated_dataset_bytes = _try_estimate_output_bytes(
+                input_logical_ops,
+            )
+
+        ray_remote_args = self._get_default_aggregator_ray_remote_args(
+            num_partitions=target_num_partitions,
+            num_aggregators=num_aggregators,
+            total_available_cluster_resources=total_available_cluster_resources,
+            estimated_dataset_bytes=estimated_dataset_bytes,
+        )
+
+        if aggregator_ray_remote_args_override is not None:
+            # Set default values missing for configs missing in the override
+            ray_remote_args.update(aggregator_ray_remote_args_override)
 
         self._aggregator_pool: AggregatorPool = AggregatorPool(
-            num_partitions=num_partitions,
+            num_partitions=target_num_partitions,
             num_aggregators=num_aggregators,
             aggregation_factory=partition_aggregation_factory,
-            aggregator_ray_remote_args=(
-                aggregator_ray_remote_args_override
-                or self._get_default_aggregator_ray_remote_args(
-                    num_partitions=num_partitions,
-                    num_aggregators=num_aggregators,
-                    partition_size_hint=partition_size_hint,
-                )
-            ),
+            aggregator_ray_remote_args=ray_remote_args,
             data_context=data_context,
         )
 
@@ -497,8 +603,10 @@ class HashShufflingOperatorBase(PhysicalOperator, HashShuffleProgressBarMixin):
         # aggregators (keeps track which input sequences have already broadcasted
         # their schemas)
         self._has_schemas_broadcasted: DefaultDict[int, bool] = defaultdict(bool)
-        # Id of the last partition finalization of which had already been scheduled
-        self._last_finalized_partition_id: int = -1
+        # Set of partitions still pending finalization
+        self._pending_finalization_partition_ids: Set[int] = set(
+            range(target_num_partitions)
+        )
 
         self._output_queue: Deque[RefBundle] = deque()
 
@@ -549,8 +657,14 @@ class HashShufflingOperatorBase(PhysicalOperator, HashShuffleProgressBarMixin):
             input_key_column_names = self._key_column_names[input_index]
             # Compose shuffling task resource bundle
             shuffle_task_resource_bundle = {
-                "num_cpus": 1,
-                "memory": self._estimate_shuffling_memory_req(block_metadata),
+                "num_cpus": 0.5,
+                "memory": self._estimate_shuffling_memory_req(
+                    block_metadata,
+                    target_max_block_size=(
+                        self._data_context.target_max_block_size
+                        or DEFAULT_TARGET_MAX_BLOCK_SIZE
+                    ),
+                ),
             }
 
             cur_shuffle_task_idx = self._next_shuffle_tasks_idx
@@ -575,6 +689,8 @@ class HashShufflingOperatorBase(PhysicalOperator, HashShuffleProgressBarMixin):
             ] = _shuffle_block.options(
                 **shuffle_task_resource_bundle,
                 num_returns=1,
+                # Make sure tasks are retried indefinitely
+                max_retries=-1,
             ).remote(
                 block_ref,
                 input_index,
@@ -621,7 +737,7 @@ class HashShufflingOperatorBase(PhysicalOperator, HashShuffleProgressBarMixin):
                 self.shuffle_metrics.on_task_finished(cur_shuffle_task_idx, None)
 
                 # Update Shuffle progress bar
-                self.shuffle_bar.update(i=input_block_metadata.num_rows)
+                self.shuffle_bar.update(increment=input_block_metadata.num_rows or 0)
 
             # TODO update metrics
             task = self._shuffling_tasks[input_index][
@@ -711,11 +827,6 @@ class HashShufflingOperatorBase(PhysicalOperator, HashShuffleProgressBarMixin):
         if not self._is_shuffling_done():
             return
 
-        logger.debug(
-            f"Scheduling next shuffling finalization batch (last finalized "
-            f"partition id is {self._last_finalized_partition_id})"
-        )
-
         def _on_bundle_ready(partition_id: int, bundle: RefBundle):
             # Add finalized block to the output queue
             self._output_queue.append(bundle)
@@ -739,7 +850,7 @@ class HashShufflingOperatorBase(PhysicalOperator, HashShuffleProgressBarMixin):
 
             # Update Finalize progress bar
             self.reduce_bar.update(
-                i=bundle.num_rows(), total=self.num_output_rows_total()
+                increment=bundle.num_rows() or 0, total=self.num_output_rows_total()
             )
 
         def _on_aggregation_done(partition_id: int, exc: Optional[Exception]):
@@ -760,10 +871,8 @@ class HashShufflingOperatorBase(PhysicalOperator, HashShuffleProgressBarMixin):
             or self._aggregator_pool.num_aggregators
         )
 
-        num_remaining_partitions = (
-            self._num_partitions - 1 - self._last_finalized_partition_id
-        )
         num_running_finalizing_tasks = len(self._finalizing_tasks)
+        num_remaining_partitions = len(self._pending_finalization_partition_ids)
 
         # Finalization is executed in batches of no more than
         # `DataContext.max_hash_shuffle_finalization_batch_size` tasks at a time.
@@ -787,12 +896,21 @@ class HashShufflingOperatorBase(PhysicalOperator, HashShuffleProgressBarMixin):
         if next_batch_size == 0:
             return
 
-        # Next partition to be scheduled for finalization is the one right
-        # after the last one scheduled
-        next_partition_id = self._last_finalized_partition_id + 1
-
-        target_partition_ids = list(
-            range(next_partition_id, next_partition_id + next_batch_size)
+        # We're sampling randomly next set of partitions to be finalized
+        # to distribute finalization window uniformly across the nodes of the cluster
+        # and avoid effect of "sliding lense" effect where we finalize the batch of
+        # N *adjacent* partitions that may be co-located on the same node:
+        #
+        #   - Adjacent partitions i and i+1 are handled by adjacent
+        #   aggregators (since membership is determined as i % num_aggregators)
+        #
+        #   - Adjacent aggregators have high likelihood of running on the
+        #   same node (when num aggregators > num nodes)
+        #
+        # NOTE: This doesn't affect determinism, since this only impacts order
+        #       of finalization (hence not required to be seeded)
+        target_partition_ids = random.sample(
+            list(self._pending_finalization_partition_ids), next_batch_size
         )
 
         logger.debug(
@@ -814,7 +932,7 @@ class HashShufflingOperatorBase(PhysicalOperator, HashShuffleProgressBarMixin):
 
             # Request finalization of the partition
             block_gen = aggregator.finalize.options(
-                **finalize_task_resource_bundle
+                **finalize_task_resource_bundle,
             ).remote(partition_id)
 
             self._finalizing_tasks[partition_id] = DataOpTask(
@@ -829,15 +947,15 @@ class HashShufflingOperatorBase(PhysicalOperator, HashShuffleProgressBarMixin):
                 ),
             )
 
+            # Pop partition id from remaining set
+            self._pending_finalization_partition_ids.remove(partition_id)
+
             # Update Finalize Metrics on task submission
             # NOTE: This is empty because the input is directly forwarded from the
             # output of the shuffling stage, which we don't return.
             empty_bundle = RefBundle([], schema=None, owns_blocks=False)
             self.reduce_metrics.on_task_submitted(partition_id, empty_bundle)
 
-        # Update last finalized partition id
-        self._last_finalized_partition_id = max(target_partition_ids)
-
     def _do_shutdown(self, force: bool = False) -> None:
         self._aggregator_pool.shutdown(force=True)
         # NOTE: It's critical for Actor Pool to release actors before calling into
@@ -909,7 +1027,7 @@ class HashShufflingOperatorBase(PhysicalOperator, HashShuffleProgressBarMixin):
         return True
 
     def _is_finalized(self):
-        return self._last_finalized_partition_id == self._num_partitions - 1
+        return len(self._pending_finalization_partition_ids) == 0
 
     def _handle_shuffled_block_metadata(
         self,
@@ -945,79 +1063,123 @@ class HashShufflingOperatorBase(PhysicalOperator, HashShuffleProgressBarMixin):
         }
 
     @classmethod
-    def _estimate_shuffling_memory_req(cls, block_metadata: BlockMetadata):
-        return (
-            math.ceil(block_metadata.size_bytes * 1.25)
-            if block_metadata.size_bytes
-            else 2 * GiB
+    def _estimate_shuffling_memory_req(
+        cls,
+        block_metadata: BlockMetadata,
+        target_max_block_size: int,
+    ):
+        estimated_block_bytes = (
+            block_metadata.size_bytes
+            if block_metadata.size_bytes is not None
+            else target_max_block_size
         )
 
+        return estimated_block_bytes * 2
+
     def _get_default_aggregator_ray_remote_args(
         self,
         *,
         num_partitions: int,
         num_aggregators: int,
-        partition_size_hint: Optional[int] = None,
+        total_available_cluster_resources: ExecutionResources,
+        estimated_dataset_bytes: Optional[int],
     ):
         assert num_partitions >= num_aggregators
-        assert partition_size_hint is None or partition_size_hint > 0
 
-        aggregator_total_memory_required = 0
-        if (
-            self.data_context.target_max_block_size is not None
-            or partition_size_hint is not None
-        ):
-            aggregator_total_memory_required = self._estimate_aggregator_memory_allocation(
+        if estimated_dataset_bytes is not None:
+            estimated_aggregator_memory_required = self._estimate_aggregator_memory_allocation(
                 num_aggregators=num_aggregators,
                 num_partitions=num_partitions,
                 # NOTE: If no partition size hint is provided we simply assume target
                 #       max block size specified as the best partition size estimate
-                partition_byte_size_estimate=(
-                    partition_size_hint or self.data_context.target_max_block_size
-                ),
+                estimated_dataset_bytes=estimated_dataset_bytes,
             )
+        else:
+            # NOTE: In cases when we're unable to estimate dataset size,
+            #       we simply fallback to request the minimum of:
+            #       - conservative 50% of total available memory for a join operation.
+            #       - ``DEFAULT_HASH_SHUFFLE_AGGREGATOR_MEMORY_ALLOCATION`` worth of
+            #       memory for every Aggregator.
 
-        # Since aggregators can handle multiple individual partitions,
-        # CPU allocation is proportionately scaled with the number of partitions
-        partition_aggregator_ratio: int = math.ceil(num_partitions / num_aggregators)
-        assert partition_aggregator_ratio >= 1
+            max_memory_per_aggregator = (
+                total_available_cluster_resources.memory / num_aggregators
+            )
+            modest_memory_per_aggregator = max_memory_per_aggregator / 2
+
+            estimated_aggregator_memory_required = min(
+                modest_memory_per_aggregator,
+                DEFAULT_HASH_SHUFFLE_AGGREGATOR_MEMORY_ALLOCATION,
+            )
 
         remote_args = {
-            "num_cpus": self._get_aggregator_num_cpus_per_partition(
-                num_partitions=num_partitions
-            )
-            * partition_aggregator_ratio,
-            "memory": aggregator_total_memory_required,
+            "num_cpus": self._get_aggregator_num_cpus(
+                total_available_cluster_resources,
+                estimated_aggregator_memory_required,
+                num_aggregators=num_aggregators,
+            ),
+            "memory": estimated_aggregator_memory_required,
             # NOTE: By default aggregating actors should be spread across available
             #       nodes to prevent any single node being overloaded with a "thundering
             #       herd"
             "scheduling_strategy": "SPREAD",
+            # Allow actor tasks to execute out of order by default to prevent head-of-line
+            # blocking scenario.
+            "allow_out_of_order_execution": True,
         }
 
         return remote_args
 
     @abc.abstractmethod
-    def _get_default_num_cpus_per_partition(self) -> int:
+    def _get_operator_num_cpus_override(self) -> int:
         pass
 
-    @abc.abstractmethod
-    def _get_operator_num_cpus_per_partition_override(self) -> int:
-        pass
+    def _get_aggregator_num_cpus(
+        self,
+        total_available_cluster_resources: ExecutionResources,
+        estimated_aggregator_memory_required: int,
+        num_aggregators: int,
+    ) -> float:
+        """Estimates number of CPU resources to be provisioned for individual
+        Aggregators.
 
-    def _get_aggregator_num_cpus_per_partition(self, num_partitions: int):
-        # 1. Check whether there is an override
-        if self._get_operator_num_cpus_per_partition_override() is not None:
-            return self._get_operator_num_cpus_per_partition_override()
+        Due to semantic of the Aggregator's role (outlined below), their CPU
+        allocation is mostly playing a role of complimenting their memory allocation
+        such that it serves as a protection mechanism from over-allocation of the
+        tasks that do not specify their respective memory resources.
+        """
 
-        # 2. Check cluster resources
-        max_resources = ray._private.state.state.get_max_resources_from_cluster_config()
-        if max_resources and (max_resources.get("CPU") or 0) > 0:
-            # NOTE: For shuffling operations we aim to allocate no more than
-            #       50% of CPUs, but no more than 1 CPU per partition
-            return min(1, (max_resources["CPU"] / 2) / num_partitions)
+        # First, check whether there is an override
+        if self._get_operator_num_cpus_override() is not None:
+            return self._get_operator_num_cpus_override()
 
-        # 3. Fallback to defaults if the first two options are not available
-        return self._get_default_num_cpus_per_partition()
+        # Note that
+        #
+        #  - Shuffle aggregators have modest computational footprint until
+        #    finalization stage
+        #  - Finalization stage actually always executes standalone, since it only
+        #    starts when all preceding operations complete
+        #
+        # Though we don't need to purposefully allocate any meaningful amount of
+        # CPU resources to the shuffle aggregators, we're still allocating nominal
+        # CPU resources to it such that to compliment its required memory allocation
+        # and therefore protect from potential OOMs in case other tasks getting
+        # scheduled onto the same node, but not specifying their respective memory
+        # requirements.
+        #
+        # CPU allocation is determined like following
+        #
+        #   CPUs = Total memory required / 4 GiB (standard ratio in the conventional clouds)
+        #
+        # But no more than
+        #   - 25% of total available CPUs but
+        #   - No more than 4 CPUs per aggregator
+        #
+        cap = min(4.0, total_available_cluster_resources.cpu * 0.25 / num_aggregators)
+
+        target_num_cpus = min(cap, estimated_aggregator_memory_required / (4 * GiB))
+
+        # Round resource to 2d decimal point (for readability)
+        return round(target_num_cpus, 2)
 
     @classmethod
     def _estimate_aggregator_memory_allocation(
@@ -1025,10 +1187,14 @@ class HashShufflingOperatorBase(PhysicalOperator, HashShuffleProgressBarMixin):
         *,
         num_aggregators: int,
         num_partitions: int,
-        partition_byte_size_estimate: int,
+        estimated_dataset_bytes: int,
     ) -> int:
         raise NotImplementedError()
 
+    @classmethod
+    def _gen_op_name(cls, num_partitions: int) -> str:
+        raise NotImplementedError()
+
 
 class HashShuffleOperator(HashShufflingOperatorBase):
     def __init__(
@@ -1037,12 +1203,14 @@ class HashShuffleOperator(HashShufflingOperatorBase):
         data_context: DataContext,
         *,
         key_columns: Tuple[str],
-        num_partitions: int,
+        num_partitions: Optional[int] = None,
         should_sort: bool = False,
         aggregator_ray_remote_args_override: Optional[Dict[str, Any]] = None,
     ):
         super().__init__(
-            name=f"Shuffle(key_columns={key_columns}, num_partitions={num_partitions})",
+            name_factory=(
+                lambda num_partitions: f"Shuffle(key_columns={key_columns}, num_partitions={num_partitions})"
+            ),
             input_ops=[input_op],
             data_context=data_context,
             key_columns=[key_columns],
@@ -1056,28 +1224,11 @@ class HashShuffleOperator(HashShufflingOperatorBase):
                     key_columns=key_columns,
                 )
             ),
-            shuffle_progress_bar_name="Shufle",
+            shuffle_progress_bar_name="Shuffle",
         )
 
-    def _get_default_num_cpus_per_partition(self) -> int:
-        """
-        CPU allocation for aggregating actors of Shuffle operator is calculated as:
-        num_cpus (per partition) = CPU budget / # partitions
-
-        Assuming:
-        - Default number of partitions: 64
-        - Total operator's CPU budget with default settings: 4 cores
-        - Number of CPUs per partition: 4 / 64 = 0.0625
-
-        These CPU budgets are derived such that Ray Data pipeline could run on a
-        single node (using the default settings).
-        """
-        return 0.0625
-
-    def _get_operator_num_cpus_per_partition_override(self) -> int:
-        return (
-            self.data_context.hash_shuffle_operator_actor_num_cpus_per_partition_override
-        )
+    def _get_operator_num_cpus_override(self) -> float:
+        return self.data_context.hash_shuffle_operator_actor_num_cpus_override
 
     @classmethod
     def _estimate_aggregator_memory_allocation(
@@ -1085,13 +1236,16 @@ class HashShuffleOperator(HashShufflingOperatorBase):
         *,
         num_aggregators: int,
         num_partitions: int,
-        partition_byte_size_estimate: int,
+        estimated_dataset_bytes: int,
     ) -> int:
-        dataset_size = num_partitions * partition_byte_size_estimate
+        partition_byte_size_estimate = math.ceil(
+            estimated_dataset_bytes / num_partitions
+        )
+
         # Estimate of object store memory required to accommodate all partitions
         # handled by a single aggregator
         aggregator_shuffle_object_store_memory_required: int = math.ceil(
-            dataset_size / num_aggregators
+            estimated_dataset_bytes / num_aggregators
         )
         # Estimate of memory required to accommodate single partition as an output
         # (inside Object Store)
@@ -1105,12 +1259,14 @@ class HashShuffleOperator(HashShufflingOperatorBase):
             output_object_store_memory_required
         )
 
-        logger.debug(
-            f"Estimated memory requirement for shuffling operator "
-            f"(partitions={num_partitions}, aggregators={num_aggregators}): "
-            f"shuffle={aggregator_shuffle_object_store_memory_required / GiB:.2f}GiB, "
-            f"output={output_object_store_memory_required / GiB:.2f}GiB, "
-            f"total={aggregator_total_memory_required / GiB:.2f}GiB, "
+        logger.info(
+            f"Estimated memory requirement for shuffling aggregator "
+            f"(partitions={num_partitions}, "
+            f"aggregators={num_aggregators}, "
+            f"dataset (estimate)={estimated_dataset_bytes / GiB:.1f}GiB): "
+            f"shuffle={aggregator_shuffle_object_store_memory_required / MiB:.1f}MiB, "
+            f"output={output_object_store_memory_required / MiB:.1f}MiB, "
+            f"total={aggregator_total_memory_required / MiB:.1f}MiB, "
         )
 
         return aggregator_total_memory_required
@@ -1167,6 +1323,11 @@ class AggregatorPool:
         # Check cluster resources before starting aggregators
         self._check_cluster_resources()
 
+        logger.debug(
+            f"Starting {self._num_aggregators} shuffle aggregators with remote "
+            f"args: {self._aggregator_ray_remote_args}"
+        )
+
         for aggregator_id in range(self._num_aggregators):
             target_partition_ids = self._aggregator_partition_map[aggregator_id]
 
@@ -1228,22 +1389,22 @@ class AggregatorPool:
             if required_memory > total_memory:
                 logger.warning(
                     f"Insufficient memory resources in cluster for hash shuffle operation. "
-                    f"Required: {required_memory / GiB:.2f} GiB for {self._num_aggregators} aggregators, "
-                    f"but cluster only has {total_memory / GiB:.2f} GiB total memory. "
+                    f"Required: {required_memory / GiB:.1f} GiB for {self._num_aggregators} aggregators, "
+                    f"but cluster only has {total_memory / GiB:.1f} GiB total memory. "
                     f"Consider reducing the number of partitions or increasing cluster size."
                 )
 
             if required_memory > available_memory:
                 logger.warning(
                     f"Limited available memory resources for hash shuffle operation. "
-                    f"Required: {required_memory / GiB:.2f} GiB, available: {available_memory / GiB:.2f} GiB. "
+                    f"Required: {required_memory / GiB:.1f} GiB, available: {available_memory / GiB:.1f} GiB. "
                     f"Aggregators may take longer to start due to resource contention."
                 )
 
             logger.debug(
                 f"Resource check passed for hash shuffle operation: "
                 f"required CPUs={required_cpus}, available CPUs={available_cpus}, "
-                f"required memory={required_memory / GiB:.2f} GiB, available memory={available_memory / GiB:.2f} GiB"
+                f"required memory={required_memory / GiB:.1f} GiB, available memory={available_memory / GiB:.1f} GiB"
             )
 
     @property
@@ -1282,6 +1443,18 @@ class AggregatorPool:
             [len(ps) for ps in aggregator_partition_map.values()]
         )
 
+        # Cap shuffle aggregator concurrency at the smaller of
+        #   - Max number of partitions per aggregator
+        #   - Threshold (8 by default)
+        max_concurrency = min(
+            max_partitions_per_aggregator,
+            DEFAULT_HASH_SHUFFLE_AGGREGATOR_MAX_CONCURRENCY,
+        )
+
+        assert (
+            max_concurrency >= 1
+        ), f"{max_partitions_per_aggregator=}, {DEFAULT_MAX_HASH_SHUFFLE_AGGREGATORS}"
+
         # NOTE: ShuffleAggregator is configured as threaded actor to allow for
         #       multiple requests to be handled "concurrently" (par GIL) --
         #       while it's not a real concurrency in its fullest of senses, having
@@ -1290,18 +1463,10 @@ class AggregatorPool:
         #       handling tasks are only blocked on GIL and are ready to execute as
         #       soon as it's released.
         finalized_remote_args = {
-            # Max concurrency is configured as a max of
-            #   - Max number of partitions allocated per aggregator
-            #   - Minimum concurrency configured
-            "max_concurrency": max(
-                max_partitions_per_aggregator,
-                HashShuffleAggregator._DEFAULT_ACTOR_MAX_CONCURRENCY,
-            ),
+            "max_concurrency": max_concurrency,
             **aggregator_ray_remote_args,
         }
 
-        logger.debug(f"Shuffle aggregator's remote args: {finalized_remote_args}")
-
         return finalized_remote_args
 
     def shutdown(self, force: bool):
@@ -1366,7 +1531,10 @@ class AggregatorPool:
         self._pending_aggregators_refs = None
 
 
-@ray.remote
+@ray.remote(
+    # Make sure tasks are retried indefinitely
+    max_task_retries=-1
+)
 class HashShuffleAggregator:
     """Actor handling of the assigned partitions during hash-shuffle operation
 
@@ -1374,10 +1542,6 @@ class HashShuffleAggregator:
           assigned partitions, and has to be thread-safe!
     """
 
-    # Default minimum value of `max_concurrency` configured
-    # for a `ShuffleAggregator` actor
-    _DEFAULT_ACTOR_MAX_CONCURRENCY = 1
-
     def __init__(
         self,
         aggregator_id: int,
@@ -1407,3 +1571,34 @@ class HashShuffleAggregator:
         # TODO break down blocks to target size
         yield block
         yield BlockMetadataWithSchema.from_block(block, stats=exec_stats)
+
+
+def _get_total_cluster_resources() -> ExecutionResources:
+    """Retrieves total available cluster resources:
+
+    1. If AutoscalerV2 is used, then corresponding max configured resources of
+        the corresponding `ClusterConfig` is returned.
+    2. In case `ClusterConfig` is not set then falls back to currently available
+        cluster resources (retrieved by `ray.cluster_resources()`)
+
+    """
+    return ExecutionResources.from_resource_dict(
+        ray._private.state.state.get_max_resources_from_cluster_config()
+        or ray.cluster_resources()
+    )
+
+
+# TODO rebase on generic operator output estimation
+def _try_estimate_output_bytes(
+    input_logical_ops: List[LogicalOperator],
+) -> Optional[int]:
+    inferred_op_output_bytes = [
+        op.infer_metadata().size_bytes for op in input_logical_ops
+    ]
+
+    # Return sum of input ops estimated output byte sizes,
+    # if all are well defined
+    if all(nbs is not None for nbs in inferred_op_output_bytes):
+        return sum(inferred_op_output_bytes)
+
+    return None
