@@ -1,10 +1,11 @@
-from typing import List
+from typing import Callable, List
 
 from .ruleset import Ruleset
 from ray.data._internal.logical.interfaces import (
     LogicalPlan,
     Optimizer,
     PhysicalPlan,
+    Plan,
     Rule,
 )
 from ray.data._internal.logical.rules.configure_map_task_memory import (
@@ -16,6 +17,7 @@ from ray.data._internal.logical.rules.inherit_target_max_block_size import (
 )
 from ray.data._internal.logical.rules.limit_pushdown import LimitPushdownRule
 from ray.data._internal.logical.rules.operator_fusion import FuseOperators
+from ray.data._internal.logical.rules.predicate_pushdown import PredicatePushdown
 from ray.data._internal.logical.rules.projection_pushdown import ProjectionPushdown
 from ray.data._internal.logical.rules.set_read_parallelism import SetReadParallelismRule
 from ray.util.annotations import DeveloperAPI
@@ -25,6 +27,7 @@ _LOGICAL_RULESET = Ruleset(
         InheritBatchFormatRule,
         LimitPushdownRule,
         ProjectionPushdown,
+        PredicatePushdown,
     ]
 )
 
@@ -65,6 +68,27 @@ class PhysicalOptimizer(Optimizer):
         return [rule_cls() for rule_cls in get_physical_ruleset()]
 
 
+def get_plan_conversion_fns() -> List[Callable[[Plan], Plan]]:
+    """Get the list of transformation functions to convert a logical plan
+    to an optimized physical plan.
+
+    This returns the 3 transformation steps:
+    1. Logical optimization
+    2. Planning (logical -> physical operators)
+    3. Physical optimization
+
+    Returns:
+        A list of transformation functions, each taking a Plan and returning a Plan.
+    """
+    from ray.data._internal.planner import create_planner
+
+    return [
+        LogicalOptimizer().optimize,  # Logical optimization
+        create_planner().plan,  # Planning
+        PhysicalOptimizer().optimize,  # Physical optimization
+    ]
+
+
 def get_execution_plan(logical_plan: LogicalPlan) -> PhysicalPlan:
     """Get the physical execution plan for the provided logical plan.
 
@@ -73,9 +97,18 @@ def get_execution_plan(logical_plan: LogicalPlan) -> PhysicalPlan:
     (2) planning: convert logical to physical operators.
     (3) physical optimization: optimize physical operators.
     """
-    from ray.data._internal.planner import create_planner
 
-    optimized_logical_plan = LogicalOptimizer().optimize(logical_plan)
+    # 1. Get planning functions
+    optimize_logical, plan, optimize_physical = get_plan_conversion_fns()
+
+    # 2. Logical -> Logical (Optimized)
+    optimized_logical_plan = optimize_logical(logical_plan)
+
+    # 3. Rewire Logical -> Logical (Optimized)
     logical_plan._dag = optimized_logical_plan.dag
-    physical_plan = create_planner().plan(optimized_logical_plan)
-    return PhysicalOptimizer().optimize(physical_plan)
+
+    # 4. Logical (Optimized) -> Physical
+    physical_plan = plan(optimized_logical_plan)
+
+    # 5. Physical (Optimized) -> Physical
+    return optimize_physical(physical_plan)
