@@ -101,7 +101,7 @@ class _CallableClassProtocol(Protocol[T, U]):
 UserDefinedFunction = Union[
     Callable[[T], U],
     Callable[[T], Iterator[U]],
-    "_CallableClassProtocol",
+    type["_CallableClassProtocol"],
 ]
 
 # A list of block references pending computation by a single task. For example,
@@ -703,8 +703,9 @@ class BlockColumnAccessor:
         Internally, Polars is used to compute row-level hashes even when the original column
         is backed by Pandas or PyArrow.
 
-        :return: A column of 64-bit integer hashes, returned in the same format as the underlying backend
-             (e.g., Pandas Series or PyArrow Array).
+        Returns:
+            A column of 64-bit integer hashes, returned in the same format as the
+            underlying backend (e.g., Pandas Series or PyArrow Array).
         """
         raise NotImplementedError()
 
@@ -720,10 +721,13 @@ class BlockColumnAccessor:
         """
         Checks whether the column is composed of list-like elements.
 
-        :param types: Optional tuple of backend-specific types to check against.
-                      If not provided, defaults to list-like types appropriate
-                      for the underlying backend (e.g., PyArrow list types).
-        :return: True if the column is made up of list-like values; False otherwise.
+        Args:
+            types: Optional tuple of backend-specific types to check against.
+                If not provided, defaults to list-like types appropriate
+                for the underlying backend (e.g., PyArrow list types).
+
+        Returns:
+            True if the column is made up of list-like values; False otherwise.
         """
         raise NotImplementedError()
 
