@@ -1,10 +1,12 @@
 import threading
 from collections import defaultdict, deque
 from dataclasses import dataclass
-from typing import Any, Dict, List, Optional, Set
+from typing import Any, Dict, List, Optional, Set, Union
 
+import ray
 import ray.util.collective as collective
 from ray._private.custom_types import TensorTransportEnum
+from ray._raylet import ObjectRef
 from ray.experimental.collective import get_tensor_transport_manager
 from ray.experimental.collective.util import device_match_transport
 from ray.util.collective.types import (
@@ -72,36 +74,56 @@ def __ray_send__(
 def __ray_recv__(
     self,
     obj_id: str,
-    tensor_transport_meta: TensorTransportMetadata,
+    tensor_transport_meta: List[Union[ObjectRef, TensorTransportMetadata]],
     communicator_meta: CommunicatorMetadata,
 ):
     """Helper function that runs on the dst actor to receive tensors from the src actor."""
     from ray._private.worker import global_worker
 
-    backend = collective.get_group_handle(communicator_meta.communicator_name).backend()
-
-    device = tensor_transport_meta.tensor_device
-    tensor_meta = tensor_transport_meta.tensor_meta
-
     gpu_object_store = global_worker.gpu_object_manager.gpu_object_store
-    if tensor_meta and not device_match_transport(device, backend):
-        raise ValueError(
-            f"Tensor transport backend {backend} does not support tensor transfer on device {device}."
+    try:
+        tensor_transport_meta: TensorTransportMetadata = (
+            ray.get(tensor_transport_meta[0])
+            if isinstance(tensor_transport_meta[0], ObjectRef)
+            else tensor_transport_meta[0]
         )
-    tensors = []
-    for meta in tensor_meta:
-        shape, dtype = meta
-        tensor = torch.empty(shape, dtype=dtype, device=device)
-        tensors.append(tensor)
+        device = tensor_transport_meta.tensor_device
+        tensor_meta = tensor_transport_meta.tensor_meta
 
+        backend = collective.get_group_handle(
+            communicator_meta.communicator_name
+        ).backend()
+
+        if tensor_meta and not device_match_transport(device, backend):
+            raise ValueError(
+                f"Tensor transport backend {backend} does not support tensor transfer on device {device}."
+            )
+
+        tensors = []
+        for meta in tensor_meta:
+            shape, dtype = meta
+            tensor = torch.empty(shape, dtype=dtype, device=device)
+            tensors.append(tensor)
+
+        tensor_transport_manager = get_tensor_transport_manager(backend)
+        tensor_transport_manager.recv_multiple_tensors(
+            tensors,
+            obj_id,
+            tensor_transport_meta,
+            communicator_meta,
+        )
+        gpu_object_store.add_object(obj_id, tensors, is_primary=False)
+    except Exception as e:
+        # Store the error as a gpu object if the recv fails,
+        # so waiters will raise the error.
+        gpu_object_store.add_object(obj_id, e, is_primary=False)
+
+
+def __ray_abort_transport__(self, obj_id: str, communicator_meta: CommunicatorMetadata):
+    """Helper function that can run on an actor doing a send or recv to abort the transport."""
+    backend = collective.get_group_handle(communicator_meta.communicator_name).backend()
     tensor_transport_manager = get_tensor_transport_manager(backend)
-    tensor_transport_manager.recv_multiple_tensors(
-        tensors,
-        tensor_transport_meta,
-        communicator_meta,
-    )
-
-    gpu_object_store.add_object(obj_id, tensors)
+    tensor_transport_manager.abort_transport(obj_id, communicator_meta)
 
 
 def __ray_free__(
@@ -145,6 +167,8 @@ class _GPUObject:
     data: List["torch.Tensor"]
     # Whether the GPU object is the primary copy.
     is_primary: bool
+    # If a recv failed, we store the error here.
+    error: Optional[Exception] = None
 
 
 class GPUObjectStore:
@@ -194,13 +218,15 @@ class GPUObjectStore:
 
     def get_object(self, obj_id: str) -> Optional[List["torch.Tensor"]]:
         with self._lock:
+            if self._gpu_object_store[obj_id][0].error:
+                raise self._gpu_object_store[obj_id][0].error
             return self._gpu_object_store[obj_id][0].data
 
     def add_object(
         self,
         obj_id: str,
-        gpu_object: List["torch.Tensor"],
-        is_primary: bool = False,
+        gpu_object: Union[List["torch.Tensor"], Exception],
+        is_primary: bool,
     ):
         """
         Add a GPU object to the GPU object store.
@@ -211,15 +237,20 @@ class GPUObjectStore:
             is_primary: Whether the GPU object is the primary copy.
         """
         with self._object_present_cv:
-            for tensor in gpu_object:
-                self._tensor_to_object_ids[tensor.data_ptr()].add(obj_id)
-            # Append to the queue instead of overwriting
-            self._gpu_object_store[obj_id].append(
-                _GPUObject(
-                    gpu_object,
-                    is_primary,
+            if isinstance(gpu_object, Exception):
+                self._gpu_object_store[obj_id].append(
+                    _GPUObject([], is_primary, error=gpu_object)
+                )
+            else:
+                for tensor in gpu_object:
+                    self._tensor_to_object_ids[tensor.data_ptr()].add(obj_id)
+                # Append to the queue instead of overwriting
+                self._gpu_object_store[obj_id].append(
+                    _GPUObject(
+                        gpu_object,
+                        is_primary,
+                    )
                 )
-            )
             self._object_present_cv.notify_all()
 
     def is_primary_copy(self, obj_id: str) -> bool:
@@ -302,9 +333,10 @@ class GPUObjectStore:
         with self._lock:
             meta = self._managed_meta_nixl.pop(obj_id)
             self._managed_meta_counts_nixl[meta] -= 1
-            if self._managed_meta_counts_nixl[meta] == 0:
+            count = self._managed_meta_counts_nixl[meta]
+            if count <= 0:
                 self._managed_meta_counts_nixl.pop(meta)
-            return self._managed_meta_counts_nixl[meta]
+            return count
 
     def wait_and_pop_object(
         self, obj_id: str, timeout: Optional[float] = None
@@ -355,6 +387,8 @@ class GPUObjectStore:
             gpu_object = queue.popleft()
             if len(queue) == 0:
                 del self._gpu_object_store[obj_id]
+            if gpu_object.error:
+                raise gpu_object.error
             for tensor in gpu_object.data:
                 self._tensor_to_object_ids[tensor.data_ptr()].remove(obj_id)
                 if len(self._tensor_to_object_ids[tensor.data_ptr()]) == 0:
