@@ -1,26 +1,28 @@
 import abc
-from collections import defaultdict
 import copy
 import logging
-import numpy
 import platform
-import tree
+from collections import defaultdict
 from typing import (
+    TYPE_CHECKING,
     Any,
     Callable,
     Collection,
     Dict,
-    List,
     Hashable,
     Iterable,
+    List,
     Optional,
     Sequence,
     Tuple,
-    TYPE_CHECKING,
     Union,
 )
 
+import numpy
+import tree
+
 import ray
+from ray._common.deprecation import Deprecated
 from ray.rllib.connectors.learner.learner_connector_pipeline import (
     LearnerConnectorPipeline,
 )
@@ -31,8 +33,8 @@ from ray.rllib.core import (
     DEFAULT_MODULE_ID,
 )
 from ray.rllib.core.learner.training_data import TrainingData
-from ray.rllib.core.rl_module.apis import SelfSupervisedLossAPI
 from ray.rllib.core.rl_module import validate_module_id
+from ray.rllib.core.rl_module.apis import SelfSupervisedLossAPI
 from ray.rllib.core.rl_module.multi_rl_module import (
     MultiRLModule,
     MultiRLModuleSpec,
@@ -41,30 +43,33 @@ from ray.rllib.core.rl_module.rl_module import RLModule, RLModuleSpec
 from ray.rllib.policy.policy import PolicySpec
 from ray.rllib.policy.sample_batch import MultiAgentBatch, SampleBatch
 from ray.rllib.utils.annotations import (
-    override,
     OverrideToImplementCustomLogic,
     OverrideToImplementCustomLogic_CallToSuperRecommended,
+    override,
 )
 from ray.rllib.utils.checkpoints import Checkpointable
 from ray.rllib.utils.debug import update_global_seed_if_necessary
-from ray._common.deprecation import Deprecated
 from ray.rllib.utils.framework import try_import_torch
 from ray.rllib.utils.metrics import (
     ALL_MODULES,
     DATASET_NUM_ITERS_TRAINED,
     DATASET_NUM_ITERS_TRAINED_LIFETIME,
+    MODULE_TRAIN_BATCH_SIZE_MEAN,
     NUM_ENV_STEPS_SAMPLED_LIFETIME,
     NUM_ENV_STEPS_TRAINED,
     NUM_ENV_STEPS_TRAINED_LIFETIME,
     NUM_MODULE_STEPS_TRAINED,
     NUM_MODULE_STEPS_TRAINED_LIFETIME,
-    MODULE_TRAIN_BATCH_SIZE_MEAN,
     WEIGHTS_SEQ_NO,
 )
 from ray.rllib.utils.metrics.metrics_logger import MetricsLogger
+from ray.rllib.utils.metrics.ray_metrics import (
+    DEFAULT_HISTOGRAM_BOUNDARIES_SHORT_EVENTS,
+    TimerAndPrometheusLogger,
+)
 from ray.rllib.utils.minibatch_utils import (
-    MiniBatchDummyIterator,
     MiniBatchCyclicIterator,
+    MiniBatchDummyIterator,
     MiniBatchRayDataIterator,
 )
 from ray.rllib.utils.schedules.scheduler import Scheduler
@@ -74,14 +79,15 @@ from ray.rllib.utils.typing import (
     ModuleID,
     Optimizer,
     Param,
-    ParamRef,
     ParamDict,
+    ParamRef,
     ResultDict,
     ShouldModuleBeUpdatedFn,
     StateDict,
     TensorType,
 )
 from ray.util.annotations import PublicAPI
+from ray.util.metrics import Counter, Histogram
 
 if TYPE_CHECKING:
     from ray.rllib.algorithms.algorithm_config import AlgorithmConfig
@@ -275,6 +281,35 @@ class Learner(Checkpointable):
         # repeatable iterator that iterates over a split of the streamed data.
         self.iterator: MiniBatchRayDataIterator = None
 
+        # Ray metrics
+        self._metrics_all_modules_num_env_steps_trained = Counter(
+            name="rllib_learner_all_modules_num_env_steps_trained_counter",
+            description="Number of env steps trained (sum over all modules).",
+            tag_keys=("rllib",),
+        )
+        self._metrics_all_modules_num_env_steps_trained.set_default_tags(
+            {"rllib": self.__class__.__name__}
+        )
+
+        self._metrics_all_modules_num_module_steps_trained = Counter(
+            name="rllib_learner_all_modules_num_module_steps_trained_counter",
+            description="Number of module steps trained (sum over all modules).",
+            tag_keys=("rllib",),
+        )
+        self._metrics_all_modules_num_module_steps_trained.set_default_tags(
+            {"rllib": self.__class__.__name__}
+        )
+
+        self._metrics_learner_inner_update = Histogram(
+            name="rllib_learner_update_inner_update_time",
+            description="Duration of the Learner's inner update.",
+            boundaries=DEFAULT_HISTOGRAM_BOUNDARIES_SHORT_EVENTS,
+            tag_keys=("rllib",),
+        )
+        self._metrics_learner_inner_update.set_default_tags(
+            {"rllib": self.__class__.__name__}
+        )
+
     # TODO (sven): Do we really need this API? It seems like LearnerGroup constructs
     #  all Learner workers and then immediately builds them any ways? Unless there is
     #  a reason related to Train worker group setup.
@@ -315,7 +350,6 @@ class Learner(Checkpointable):
 
         # Log the number of trainable/non-trainable parameters.
         self._log_trainable_parameters()
-
         self._is_built = True
 
     @property
@@ -809,6 +843,7 @@ class Learner(Checkpointable):
             module_id=module_id,
             config=self.config.get_config_for_module(module_id),
         )
+
         return self.config.rl_module_spec
 
     @OverrideToImplementCustomLogic_CallToSuperRecommended
@@ -1073,7 +1108,13 @@ class Learner(Checkpointable):
 
             # Make the actual in-graph/traced `_update` call. This should return
             # all tensor values (no numpy).
-            fwd_out, loss_per_module, _ = self._update(tensor_minibatch.policy_batches)
+            with TimerAndPrometheusLogger(self._metrics_learner_inner_update):
+                fwd_out, loss_per_module, _ = self._update(
+                    tensor_minibatch.policy_batches
+                )
+
+            # Ray metrics
+            self._log_metrics(batch=tensor_minibatch)
 
             # TODO (sven): Maybe move this into loop above to get metrics more accuratcely
             #  cover the minibatch/epoch logic.
@@ -1619,6 +1660,10 @@ class Learner(Checkpointable):
 
     def _log_steps_trained_metrics(self, batch: MultiAgentBatch):
         """Logs this iteration's steps trained, based on given `batch`."""
+        # Collect all module steps and add them for `ALL_MODULES` to avoid
+        # biasing the throughput by looping through modules.
+        total_module_steps = 0
+        # Loop through all modules.
         for mid, module_batch in batch.policy_batches.items():
             # Log weights seq no for this batch.
             self.metrics.log_value(
@@ -1646,20 +1691,21 @@ class Learner(Checkpointable):
                 reduce="sum",
                 with_throughput=True,
             )
-            # Log module steps (sum of all modules).
-            self.metrics.log_value(
-                key=(ALL_MODULES, NUM_MODULE_STEPS_TRAINED),
-                value=module_batch_size,
-                reduce="sum",
-                clear_on_reduce=True,
-                with_throughput=True,
-            )
-            self.metrics.log_value(
-                key=(ALL_MODULES, NUM_MODULE_STEPS_TRAINED_LIFETIME),
-                value=module_batch_size,
-                reduce="sum",
-                with_throughput=True,
-            )
+            total_module_steps += module_batch_size
+
+        # Log module steps (sum of all modules).
+        self.metrics.log_value(
+            key=(ALL_MODULES, NUM_MODULE_STEPS_TRAINED),
+            value=total_module_steps,
+            reduce="sum",
+            clear_on_reduce=True,
+        )
+        self.metrics.log_value(
+            key=(ALL_MODULES, NUM_MODULE_STEPS_TRAINED_LIFETIME),
+            value=total_module_steps,
+            reduce="sum",
+            with_throughput=True,
+        )
         # Log env steps (all modules).
         self.metrics.log_value(
             (ALL_MODULES, NUM_ENV_STEPS_TRAINED),
@@ -1699,3 +1745,21 @@ class Learner(Checkpointable):
     @Deprecated(new="Learner.compute_losses(...)", error=True)
     def compute_loss(self, *args, **kwargs):
         pass
+
+    def _log_metrics(self, batch: MultiAgentBatch) -> None:
+        _env_steps = int(batch.env_steps())
+        if _env_steps > 0:
+            self._metrics_all_modules_num_env_steps_trained.inc(value=_env_steps)
+            total_module_steps = sum(
+                len(module_batch) for module_batch in batch.policy_batches.values()
+            )
+            self._metrics_all_modules_num_module_steps_trained.inc(
+                value=total_module_steps
+            )
+        else:
+            logger.warning(
+                f"RLlib {self.__class__.__name__}: Skipping Prometheus logging for metrics: "
+                f"{self._metrics_all_modules_num_env_steps_trained.info['name']} and "
+                f"{self._metrics_all_modules_num_module_steps_trained.info['name']}. "
+                f"Received MultiAgentBatch.env_steps()={_env_steps}, but the number of steps must be greater than 0."
+            )
