@@ -1,14 +1,14 @@
 import copy
-import gymnasium as gym
 import logging
-from typing import Callable, Dict, List, Tuple, Optional, Union, Set, Type
+from typing import Callable, Dict, List, Optional, Set, Tuple, Type, Union
 
+import gymnasium as gym
 import numpy as np
 
+from ray._common.deprecation import Deprecated
 from ray.rllib.env.base_env import BaseEnv
 from ray.rllib.env.env_context import EnvContext
 from ray.rllib.utils.annotations import OldAPIStack, override
-from ray._common.deprecation import Deprecated
 from ray.rllib.utils.typing import (
     AgentID,
     EnvCreator,
@@ -250,8 +250,7 @@ class MultiAgentEnv(gym.Env):
 
         """
 
-        from ray.rllib.env.wrappers.group_agents_wrapper import \
-            GroupAgentsWrapper
+        from ray.rllib.env.wrappers.group_agents_wrapper import GroupAgentsWrapper
         return GroupAgentsWrapper(self, groups, obs_space, act_space)
 
     # __grouping_doc_end__
