@@ -1,19 +1,20 @@
-from collections import defaultdict
 import copy
 import functools
-import numpy as np
 import time
 import uuid
-
-import gymnasium as gym
-from gymnasium.core import ActType, ObsType
+from collections import defaultdict
 from typing import Any, Dict, List, Optional, SupportsFloat, Union
 
+import gymnasium as gym
+import numpy as np
+import tree
+from gymnasium.core import ActType, ObsType
+
+from ray._common.deprecation import Deprecated
 from ray.rllib.core.columns import Columns
 from ray.rllib.env.utils.infinite_lookback_buffer import InfiniteLookbackBuffer
 from ray.rllib.policy.sample_batch import SampleBatch
 from ray.rllib.utils.serialization import gym_space_from_dict, gym_space_to_dict
-from ray._common.deprecation import Deprecated
 from ray.rllib.utils.typing import AgentID, ModuleID
 from ray.util.annotations import PublicAPI
 
@@ -238,12 +239,12 @@ class SingleAgentEpisode:
                 this parameter is provided the episode starts at the provided value.
             len_lookback_buffer: The size of the (optional) lookback buffers to keep in
                 front of this Episode for each type of data (observations, actions,
-                etc..). If larger 0, will interpret the first `len_lookback_buffer`
-                items in each type of data as NOT part of this actual
+                etc..). If larger than 0, the first `len_lookback_buffer`
+                items of each type of data are interpreted as NOT part of this actual
                 episode chunk, but instead serve as "historical" record that may be
                 viewed and used to derive new data from. For example, it might be
                 necessary to have a lookback buffer of four if you would like to do
-                observation frame stacking and your episode has been cut and you are now
+                observation frame stacking and your episode has been cut and you're now
                 operating on a new chunk (continuing from the cut one). Then, for the
                 first 3 items, you would have to be able to look back into the old
                 chunk's data.
@@ -283,6 +284,7 @@ class SingleAgentEpisode:
                 lookback=len_lookback_buffer,
             )
         self.observation_space = observation_space
+
         # Infos: t0 (initial info) to T.
         if isinstance(infos, InfiniteLookbackBuffer):
             self.infos = infos
@@ -291,6 +293,7 @@ class SingleAgentEpisode:
                 data=infos,
                 lookback=len_lookback_buffer,
             )
+
         # Actions: t1 to T.
         self._action_space = None
         if isinstance(actions, InfiniteLookbackBuffer):
@@ -301,6 +304,7 @@ class SingleAgentEpisode:
                 lookback=len_lookback_buffer,
             )
         self.action_space = action_space
+
         # Rewards: t1 to T.
         if isinstance(rewards, InfiniteLookbackBuffer):
             self.rewards = rewards
@@ -616,7 +620,17 @@ class SingleAgentEpisode:
         other.validate()
 
         # Make sure, end matches other episode chunk's beginning.
-        assert np.all(other.observations[0] == self.observations[-1])
+        tree.assert_same_structure(other.observations[0], self.observations[-1])
+        # Use tree.map_structure with np.array_equal to check every leaf node are equivalent
+        #   then np.all on flatten to validate all are tree
+        assert np.all(
+            tree.flatten(
+                tree.map_structure(
+                    np.array_equal, other.observations[0], self.observations[-1]
+                )
+            )
+        )
+
         # Pop out our last observations and infos (as these are identical
         # to the first obs and infos in the next episode).
         self.observations.pop()
