@@ -1,25 +1,25 @@
-from collections import namedtuple, OrderedDict
-import gymnasium as gym
 import logging
 import re
-import tree  # pip install dm_tree
+from collections import OrderedDict, namedtuple
 from typing import Callable, Dict, List, Optional, Tuple, Type, Union
 
-from ray.util.debug import log_once
-from ray.rllib.models.tf.tf_action_dist import TFActionDistribution
+import gymnasium as gym
+import tree  # pip install dm_tree
+
+from ray._common.deprecation import (
+    DEPRECATED_VALUE,
+    deprecation_warning,
+)
+from ray.rllib.models.catalog import ModelCatalog
 from ray.rllib.models.modelv2 import ModelV2
+from ray.rllib.models.tf.tf_action_dist import TFActionDistribution
 from ray.rllib.policy.policy import Policy
 from ray.rllib.policy.sample_batch import SampleBatch
 from ray.rllib.policy.tf_policy import TFPolicy
 from ray.rllib.policy.view_requirement import ViewRequirement
-from ray.rllib.models.catalog import ModelCatalog
 from ray.rllib.utils import force_list
 from ray.rllib.utils.annotations import OldAPIStack, override
 from ray.rllib.utils.debug import summarize
-from ray._common.deprecation import (
-    deprecation_warning,
-    DEPRECATED_VALUE,
-)
 from ray.rllib.utils.framework import try_import_tf
 from ray.rllib.utils.metrics import (
     DIFF_NUM_GRAD_UPDATES_VS_SAMPLER_POLICY,
@@ -28,11 +28,12 @@ from ray.rllib.utils.metrics import (
 from ray.rllib.utils.spaces.space_utils import get_dummy_batch_for_space
 from ray.rllib.utils.tf_utils import get_placeholder
 from ray.rllib.utils.typing import (
+    AlgorithmConfigDict,
     LocalOptimizer,
     ModelGradients,
     TensorType,
-    AlgorithmConfigDict,
 )
+from ray.util.debug import log_once
 
 tf1, tf, tfv = try_import_tf()
 
