@@ -18,12 +18,12 @@ from ray.rllib.models.torch.torch_modelv2 import TorchModelV2
 from ray.rllib.policy.policy import Policy
 from ray.rllib.policy.sample_batch import SampleBatch
 from ray.rllib.policy.torch_policy import TorchPolicy
-from ray.rllib.utils import add_mixins, NullContextManager
+from ray.rllib.utils import NullContextManager, add_mixins
 from ray.rllib.utils.annotations import OldAPIStack, override
-from ray.rllib.utils.framework import try_import_torch, try_import_jax
+from ray.rllib.utils.framework import try_import_jax, try_import_torch
 from ray.rllib.utils.metrics.learner_info import LEARNER_STATS_KEY
 from ray.rllib.utils.numpy import convert_to_numpy
-from ray.rllib.utils.typing import ModelGradients, TensorType, AlgorithmConfigDict
+from ray.rllib.utils.typing import AlgorithmConfigDict, ModelGradients, TensorType
 
 jax, _ = try_import_jax()
 torch, _ = try_import_torch()
