@@ -1,8 +1,6 @@
-from ray._private.runtime_env.mpi import mpi_init  # noqa: E402,F401
 from ray.runtime_env.runtime_env import RuntimeEnv, RuntimeEnvConfig  # noqa: E402,F401
 
 __all__ = [
     "RuntimeEnvConfig",
     "RuntimeEnv",
-    "mpi_init",
 ]
