@@ -26,12 +26,16 @@ import ray._private.services as services
 from ray._common.network_utils import build_address, parse_address
 from ray._common.usage import usage_lib
 from ray._common.utils import load_class
+from ray._private.authentication.authentication_token_setup import (
+    ensure_token_if_auth_enabled,
+)
 from ray._private.internal_api import memory_summary
 from ray._private.label_utils import (
     parse_node_labels_from_yaml_file,
     parse_node_labels_json,
     parse_node_labels_string,
 )
+from ray._private.log import format_returncode, setup_process_exit_logger
 from ray._private.resource_isolation_config import ResourceIsolationConfig
 from ray._private.utils import (
     get_ray_client_dependency_error,
@@ -526,7 +530,8 @@ Windows powershell users need additional escaping:
     "--block",
     is_flag=True,
     default=False,
-    help="provide this argument to block forever in this command",
+    help="provide this argument to block forever in this command."
+    "Process exit logs will be saved to ray_process_exit.log in the logs directory.",
 )
 @click.option(
     "--plasma-directory",
@@ -648,7 +653,7 @@ Windows powershell users need additional escaping:
     "--system-reserved-cpu",
     required=False,
     type=float,
-    help="The number of cpu cores to reserve for ray system processes. "
+    help=" The number of cpu cores to reserve for ray system processes. "
     "Cores can be fractional i.e. 1.5 means one and a half a cpu core. "
     "By default, the value will be atleast 1 core, and at maximum 3 cores. The default value "
     "is calculated using the formula min(3.0, max(1.0, 0.05 * num_cores_on_the_system)) "
@@ -937,6 +942,14 @@ def start(
                     " flag of `ray start` command."
                 )
 
+        # Ensure auth token is available if authentication mode is token
+        try:
+            ensure_token_if_auth_enabled(system_config, create_token_if_missing=False)
+        except ray.exceptions.AuthenticationError:
+            raise RuntimeError(
+                "Failed to load authentication token. To generate a token for local development, use `ray get-auth-token --generate`. For remote clusters, ensure that the token is propagated to all nodes of the cluster when token authentication is enabled."
+            )
+
         node = ray._private.node.Node(
             ray_params, head=True, shutdown_at_exit=block, spawn_reaper=block
         )
@@ -1094,6 +1107,9 @@ def start(
 
         cli_logger.labeled_value("Local node IP", ray_params.node_ip_address)
 
+        # Ensure auth token is available if authentication mode is token
+        ensure_token_if_auth_enabled(system_config, create_token_if_missing=False)
+
         node = ray._private.node.Node(
             ray_params, head=False, shutdown_at_exit=block, spawn_reaper=block
         )
@@ -1117,6 +1133,8 @@ def start(
     ray._private.utils.write_ray_address(ray_params.gcs_address, temp_dir)
 
     if block:
+        logs_dir = node.get_logs_dir_path()
+        process_exit_log_path = os.path.join(logs_dir, "ray_process_exit.log")
         cli_logger.newline()
         with cli_logger.group(cf.bold("--block")):
             cli_logger.print(
@@ -1127,7 +1145,15 @@ def start(
                 "printed if any of them terminate unexpectedly. Subprocesses "
                 "exit with SIGTERM will be treated as graceful, thus NOT reported."
             )
+            cli_logger.print(
+                "Process exit logs will be saved to: {}", cf.bold(process_exit_log_path)
+            )
             cli_logger.flush()
+            try:
+                process_exit_logger = setup_process_exit_logger(process_exit_log_path)
+            except Exception as e:
+                cli_logger.warning("Failed to init process exit logger: {}", e)
+                process_exit_logger = None
 
         while True:
             time.sleep(1)
@@ -1154,6 +1180,7 @@ def start(
                 cli_logger.newline()
                 cli_logger.error("Some Ray subprocesses exited unexpectedly:")
 
+                lines_for_file = []
                 with cli_logger.indented():
                     for process_type, process in unexpected_deceased:
                         cli_logger.error(
@@ -1161,9 +1188,21 @@ def start(
                             cf.bold(str(process_type)),
                             _tags={"exit code": str(process.returncode)},
                         )
+                        rc = getattr(process, "returncode", None)
+                        rc_str = format_returncode(rc)
+                        lines_for_file.append(f"  {process_type} [exit code={rc_str}]")
+                try:
+                    file_msg = (
+                        "Some Ray subprocesses exited unexpectedly:\n"
+                        + "\n".join(lines_for_file)
+                    )
+                    process_exit_logger.error("%s", file_msg)
+                except Exception as e:
+                    cli_logger.warning("Failed to write process exit log: {}", e)
 
                 cli_logger.newline()
                 cli_logger.error("Remaining processes will be killed.")
+
                 # explicitly kill all processes since atexit handlers
                 # will not exit with errors.
                 node.kill_all_processes(check_alive=False, allow_graceful=False)
@@ -2599,8 +2638,8 @@ def install_nightly(verbose, dryrun):
 def cpp(show_library_path, generate_bazel_project_template_to):
     """Show the cpp library path and generate the bazel project template."""
     if sys.platform == "win32":
-        cli_logger.error("Ray C++ API is not supported on Windows currently.")
-        sys.exit(1)
+        raise click.ClickException("Ray C++ API is not supported on Windows currently.")
+
     if not show_library_path and not generate_bazel_project_template_to:
         raise ValueError(
             "Please input at least one option of '--show-library-path'"
@@ -2677,6 +2716,55 @@ def shutdown_prometheus():
         sys.exit(1)
 
 
+@cli.command(name="get-auth-token")
+@click.option(
+    "--generate",
+    is_flag=True,
+    default=False,
+    help="Generate a new token if none exists",
+)
+def get_auth_token(generate):
+    """Prints the Ray authentication token to stdout when RAY_AUTH_MODE=token.
+
+    If --generate is specified, a new token is created and saved to ~/.ray/auth_token if one does not exist.
+    """
+    from ray._private.authentication.authentication_token_setup import (
+        generate_and_save_token,
+    )
+    from ray._raylet import (
+        AuthenticationMode,
+        AuthenticationTokenLoader,
+        get_authentication_mode,
+    )
+
+    # Check if token auth mode is enabled and provide guidance if not
+    if get_authentication_mode() != AuthenticationMode.TOKEN:
+        raise click.ClickException(
+            "Token authentication is not currently enabled. To enable token authentication, set: export RAY_AUTH_MODE=token\n For more instructions, see: https://docs.ray.io/en/latest/ray-security/auth.html",
+        )
+
+    # Try to load existing token
+    loader = AuthenticationTokenLoader.instance()
+
+    if not loader.has_token():
+        if generate:
+            click.echo("Generating new authentication token...", err=True)
+            generate_and_save_token()
+            loader.reset_cache()
+        else:
+            raise click.ClickException(
+                "No authentication token found. Use ray `get-auth-token --generate` to create one.",
+            )
+
+    # Get raw token value
+    token = loader.get_raw_token()
+
+    # Print token to stdout (for piping) without newline
+    click.echo(token, nl=False)
+    # Print newline to stderr for clean terminal display (doesn't affect piping)
+    click.echo("", err=True)
+
+
 def add_command_alias(command, name, hidden):
     new_command = copy.deepcopy(command)
     new_command.hidden = hidden
@@ -2717,6 +2805,7 @@ cli.add_command(drain_node)
 cli.add_command(check_open_ports)
 cli.add_command(sanity_check)
 cli.add_command(symmetric_run, name="symmetric-run")
+cli.add_command(get_auth_token)
 
 try:
     from ray.util.state.state_cli import (
