@@ -1,18 +1,17 @@
 import logging
 import time
 from collections import defaultdict
-from dataclasses import dataclass
-from typing import Any, Dict, List, Optional, Set
+from typing import Any, Callable, Dict, List, Optional, Set, Tuple
 
 from ray.serve._private.common import (
-    ONGOING_REQUESTS_KEY,
     RUNNING_REQUESTS_KEY,
+    ApplicationName,
     DeploymentID,
     HandleMetricReport,
     ReplicaID,
     ReplicaMetricReport,
     TargetCapacityDirection,
-    TimeStampedValue,
+    TimeSeries,
 )
 from ray.serve._private.constants import (
     RAY_SERVE_AGGREGATE_METRICS_AT_CONTROLLER,
@@ -21,54 +20,17 @@ from ray.serve._private.constants import (
 )
 from ray.serve._private.deployment_info import DeploymentInfo
 from ray.serve._private.metrics_utils import (
-    merge_timeseries_dicts,
-    time_weighted_average,
+    aggregate_timeseries,
+    merge_instantaneous_total,
 )
+from ray.serve._private.usage import ServeUsageTag
 from ray.serve._private.utils import get_capacity_adjusted_num_replicas
+from ray.serve.config import AutoscalingContext, AutoscalingPolicy
 
 logger = logging.getLogger(SERVE_LOGGER_NAME)
 
 
-@dataclass
-class AutoscalingContext:
-    """Rich context provided to custom autoscaling policies."""
-
-    # Deployment information
-    deployment_id: DeploymentID
-    deployment_name: str
-    app_name: Optional[str]
-
-    # Current state
-    current_num_replicas: int
-    target_num_replicas: int
-    running_replicas: List[ReplicaID]
-
-    # Built-in metrics
-    total_num_requests: float
-    queued_requests: Optional[float]
-    requests_per_replica: Dict[ReplicaID, float]
-
-    # Custom metrics
-    aggregated_metrics: Dict[str, Dict[ReplicaID, float]]
-    raw_metrics: Dict[str, Dict[ReplicaID, List[float]]]
-
-    # Capacity and bounds
-    capacity_adjusted_min_replicas: int
-    capacity_adjusted_max_replicas: int
-
-    # Policy state
-    policy_state: Dict[str, Any]
-
-    # Timing
-    last_scale_up_time: Optional[float]
-    last_scale_down_time: Optional[float]
-    current_time: Optional[float]
-
-    # Config
-    config: Optional[Any]
-
-
-class AutoscalingState:
+class DeploymentAutoscalingState:
     """Manages autoscaling for a single deployment."""
 
     def __init__(self, deployment_id: DeploymentID):
@@ -85,7 +47,12 @@ class AutoscalingState:
 
         self._deployment_info = None
         self._config = None
-        self._policy = None
+        self._policy: Optional[
+            Callable[[AutoscalingContext], Tuple[int, Optional[Dict[str, Any]]]]
+        ] = None
+        # user defined policy returns a dictionary of state that is persisted between autoscaling decisions
+        # content of the dictionary is determined by the user defined policy
+        self._policy_state: Optional[Dict[str, Any]] = None
         self._running_replicas: List[ReplicaID] = []
         self._target_capacity: Optional[float] = None
         self._target_capacity_direction: Optional[TargetCapacityDirection] = None
@@ -97,6 +64,10 @@ class AutoscalingState:
         """
 
         config = info.deployment_config.autoscaling_config
+        if config is None:
+            raise ValueError(
+                f"Autoscaling config is not set for deployment {self._deployment_id}"
+            )
         if (
             self._deployment_info is None or self._deployment_info.config_changed(info)
         ) and config.initial_replicas is not None:
@@ -106,11 +77,20 @@ class AutoscalingState:
 
         self._deployment_info = info
         self._config = config
-        self._policy = self._config.get_policy()
+        self._policy = self._config.policy.get_policy()
         self._target_capacity = info.target_capacity
         self._target_capacity_direction = info.target_capacity_direction
         self._policy_state = {}
 
+        # Log when custom autoscaling policy is used for deployment
+        if not self._config.policy.is_default_policy_function():
+            logger.info(
+                f"Using custom autoscaling policy '{self._config.policy.policy_function}' "
+                f"for deployment '{self._deployment_id}'."
+            )
+            # Record telemetry for custom autoscaling policy usage
+            ServeUsageTag.CUSTOM_AUTOSCALING_POLICY_USED.record("1")
+
         return self.apply_bounds(target_num_replicas)
 
     def on_replica_stopped(self, replica_id: ReplicaID):
@@ -245,6 +225,21 @@ class AutoscalingState:
         and max adjusted by the target capacity and returned. If
         `_skip_bound_check` is True, then the bounds are not applied.
         """
+        if self._policy is None:
+            raise ValueError(f"Policy is not set for deployment {self._deployment_id}.")
+        autoscaling_context = self.get_autoscaling_context(curr_target_num_replicas)
+        decision_num_replicas, self._policy_state = self._policy(autoscaling_context)
+        if _skip_bound_check:
+            return decision_num_replicas
+
+        return self.apply_bounds(decision_num_replicas)
+
+    def get_autoscaling_context(self, curr_target_num_replicas):
+        total_num_requests = self.get_total_num_requests()
+        total_queued_requests = self._get_queued_requests()
+        # NOTE: for non additive aggregation functions, total_running_requests is not
+        # accurate, consider this is a approximation.
+        total_running_requests = total_num_requests - total_queued_requests
 
         autoscaling_context: AutoscalingContext = AutoscalingContext(
             deployment_id=self._deployment_id,
@@ -253,32 +248,29 @@ class AutoscalingState:
             current_num_replicas=len(self._running_replicas),
             target_num_replicas=curr_target_num_replicas,
             running_replicas=self._running_replicas,
-            total_num_requests=self.get_total_num_requests(),
+            total_num_requests=total_num_requests,
             capacity_adjusted_min_replicas=self.get_num_replicas_lower_bound(),
             capacity_adjusted_max_replicas=self.get_num_replicas_upper_bound(),
-            policy_state=self._policy_state.copy(),
+            policy_state=(
+                self._policy_state.copy() if self._policy_state is not None else {}
+            ),
             current_time=time.time(),
             config=self._config,
-            queued_requests=None,
-            requests_per_replica=None,
-            aggregated_metrics=None,
-            raw_metrics=None,
+            total_queued_requests=total_queued_requests,
+            total_running_requests=total_running_requests,
+            aggregated_metrics=self._get_aggregated_custom_metrics(),
+            raw_metrics=self._get_raw_custom_metrics(),
             last_scale_up_time=None,
             last_scale_down_time=None,
         )
 
-        decision_num_replicas, self._policy_state = self._policy(autoscaling_context)
+        return autoscaling_context
 
-        if _skip_bound_check:
-            return decision_num_replicas
-
-        return self.apply_bounds(decision_num_replicas)
-
-    def _collect_replica_running_requests(self) -> List[List[TimeStampedValue]]:
+    def _collect_replica_running_requests(self) -> List[TimeSeries]:
         """Collect running requests timeseries from replicas for aggregation.
 
         Returns:
-            List of timeseries data (List[TimeStampedValue]).
+            List of timeseries data.
         """
         timeseries_list = []
 
@@ -294,22 +286,22 @@ class AutoscalingState:
 
         return timeseries_list
 
-    def _collect_handle_queued_requests(self) -> List[List[TimeStampedValue]]:
+    def _collect_handle_queued_requests(self) -> List[TimeSeries]:
         """Collect queued requests timeseries from all handles.
 
         Returns:
-            List of timeseries data (List[TimeStampedValue]).
+            List of timeseries data.
         """
         timeseries_list = []
         for handle_metric_report in self._handle_requests.values():
             timeseries_list.append(handle_metric_report.queued_requests)
         return timeseries_list
 
-    def _collect_handle_running_requests(self) -> List[List[TimeStampedValue]]:
+    def _collect_handle_running_requests(self) -> List[TimeSeries]:
         """Collect running requests timeseries from handles when not collected on replicas.
 
         Returns:
-            List of timeseries data (List[TimeStampedValue]).
+            List of timeseries data.
 
         Example:
             If there are 2 handles, each managing 2 replicas, and the running requests metrics are:
@@ -339,49 +331,44 @@ class AutoscalingState:
 
         return timeseries_list
 
-    def _aggregate_ongoing_requests(
-        self, metrics_timeseries_dicts: List[Dict[str, List[TimeStampedValue]]]
+    def _merge_and_aggregate_timeseries(
+        self,
+        timeseries_list: List[TimeSeries],
     ) -> float:
-        """Aggregate and average ongoing requests from timeseries data using instantaneous merge.
+        """Aggregate and average a metric from timeseries data using instantaneous merge.
 
         Args:
-            metrics_timeseries_dicts: A list of dictionaries, each containing a key-value pair:
-                - The key is the name of the metric (ONGOING_REQUESTS_KEY)
-                - The value is a list of TimeStampedValue objects, each representing a single measurement of the metric
-                this list is sorted by timestamp ascending
+            timeseries_list: A list of TimeSeries (TimeSeries), where each
+                TimeSeries represents measurements from a single source (replica, handle, etc.).
+                Each list is sorted by timestamp ascending.
 
         Returns:
-            The time-weighted average of the ongoing requests
+            The time-weighted average of the metric
 
         Example:
-            If the metrics_timeseries_dicts is:
+            If the timeseries_list is:
             [
-                {
-                    "ongoing_requests": [
-                        TimeStampedValue(timestamp=0.1, value=5.0),
-                        TimeStampedValue(timestamp=0.2, value=7.0),
-                    ]
-                },
-                {
-                    "ongoing_requests": [
-                        TimeStampedValue(timestamp=0.2, value=3.0),
-                        TimeStampedValue(timestamp=0.3, value=1.0),
-                    ]
-                }
+                [
+                    TimeStampedValue(timestamp=0.1, value=5.0),
+                    TimeStampedValue(timestamp=0.2, value=7.0),
+                ],
+                [
+                    TimeStampedValue(timestamp=0.2, value=3.0),
+                    TimeStampedValue(timestamp=0.3, value=1.0),
+                ]
             ]
             Then the returned value will be:
             (5.0*0.1 + 7.0*0.2 + 3.0*0.2 + 1.0*0.3) / (0.1 + 0.2 + 0.2 + 0.3) = 4.5 / 0.8 = 5.625
         """
 
-        if not metrics_timeseries_dicts:
+        if not timeseries_list:
             return 0.0
 
         # Use instantaneous merge approach - no arbitrary windowing needed
-        aggregated_metrics = merge_timeseries_dicts(*metrics_timeseries_dicts)
-        ongoing_requests_timeseries = aggregated_metrics.get(ONGOING_REQUESTS_KEY, [])
-        if ongoing_requests_timeseries:
+        merged_timeseries = merge_instantaneous_total(timeseries_list)
+        if merged_timeseries:
             # assume that the last recorded metric is valid for last_window_s seconds
-            last_metric_time = ongoing_requests_timeseries[-1].timestamp
+            last_metric_time = merged_timeseries[-1].timestamp
             # we dont want to make any assumption about how long the last metric will be valid
             # only conclude that the last metric is valid for last_window_s seconds that is the
             # difference between the current time and the last metric recorded time
@@ -390,11 +377,13 @@ class AutoscalingState:
             # between replicas and controller. Also add a small epsilon to avoid division by zero
             if last_window_s <= 0:
                 last_window_s = 1e-3
-            # Calculate the time-weighted average of the running requests
-            avg_ongoing = time_weighted_average(
-                ongoing_requests_timeseries, last_window_s=last_window_s
+            # Calculate the aggregated metric value
+            value = aggregate_timeseries(
+                merged_timeseries,
+                aggregation_function=self._config.aggregation_function,
+                last_window_s=last_window_s,
             )
-            return avg_ongoing if avg_ongoing is not None else 0.0
+            return value if value is not None else 0.0
 
         return 0.0
 
@@ -460,11 +449,11 @@ class AutoscalingState:
             Total number of requests (average running + queued) calculated from
             timeseries data aggregation.
         """
-        # Collect replica-based running requests (returns List[List[TimeStampedValue]])
+        # Collect replica-based running requests (returns List[TimeSeries])
         replica_timeseries = self._collect_replica_running_requests()
         metrics_collected_on_replicas = len(replica_timeseries) > 0
 
-        # Collect queued requests from handles (returns List[List[TimeStampedValue]])
+        # Collect queued requests from handles (returns List[TimeSeries])
         queued_timeseries = self._collect_handle_queued_requests()
 
         if not metrics_collected_on_replicas:
@@ -473,23 +462,23 @@ class AutoscalingState:
         else:
             handle_timeseries = []
 
-        # Create minimal dictionary objects only when needed
-        ongoing_requests_metrics = []
+        # Collect all timeseries for ongoing requests
+        ongoing_requests_timeseries = []
 
-        # Add replica timeseries with minimal dict wrapping
-        for timeseries in replica_timeseries:
-            ongoing_requests_metrics.append({ONGOING_REQUESTS_KEY: timeseries})
+        # Add replica timeseries
+        ongoing_requests_timeseries.extend(replica_timeseries)
 
         # Add handle timeseries if replica metrics weren't collected
         if not metrics_collected_on_replicas:
-            for timeseries in handle_timeseries:
-                ongoing_requests_metrics.append({ONGOING_REQUESTS_KEY: timeseries})
+            ongoing_requests_timeseries.extend(handle_timeseries)
+
+        # Add queued timeseries
+        ongoing_requests_timeseries.extend(queued_timeseries)
 
-        # Add queued timeseries with minimal dict wrapping
-        for timeseries in queued_timeseries:
-            ongoing_requests_metrics.append({ONGOING_REQUESTS_KEY: timeseries})
         # Aggregate and add running requests to total
-        ongoing_requests = self._aggregate_ongoing_requests(ongoing_requests_metrics)
+        ongoing_requests = self._merge_and_aggregate_timeseries(
+            ongoing_requests_timeseries
+        )
 
         return ongoing_requests
 
@@ -578,11 +567,8 @@ class AutoscalingState:
         else:
             return self._calculate_total_requests_simple_mode()
 
-    def get_replica_metrics(self, agg_func: str) -> Dict[ReplicaID, List[Any]]:
+    def get_replica_metrics(self) -> Dict[ReplicaID, List[TimeSeries]]:
         """Get the raw replica metrics dict."""
-        # arcyleung TODO: pass agg_func from autoscaling policy https://github.com/ray-project/ray/pull/51905
-        # Dummy implementation of mean agg_func across all values of the same metrics key
-
         metric_values = defaultdict(list)
         for id in self._running_replicas:
             if id in self._replica_metrics and self._replica_metrics[id].metrics:
@@ -591,16 +577,294 @@ class AutoscalingState:
 
         return metric_values
 
+    def _get_queued_requests(self) -> float:
+        """Calculate the total number of queued requests across all handles.
+
+        Returns:
+            Sum of queued requests at all handles. Uses aggregated values in simple mode,
+            or aggregates timeseries data in aggregate mode.
+        """
+        if RAY_SERVE_AGGREGATE_METRICS_AT_CONTROLLER:
+            # Aggregate mode: collect and aggregate timeseries
+            queued_timeseries = self._collect_handle_queued_requests()
+            if not queued_timeseries:
+                return 0.0
+
+            return self._merge_and_aggregate_timeseries(queued_timeseries)
+        else:
+            # Simple mode: sum pre-aggregated values
+            return sum(
+                handle_metric.aggregated_queued_requests
+                for handle_metric in self._handle_requests.values()
+            )
+
+    def _get_aggregated_custom_metrics(self) -> Dict[str, Dict[ReplicaID, float]]:
+        """Aggregate custom metrics from replica metric reports.
+
+        This method aggregates raw timeseries data from replicas on the controller,
+        similar to how ongoing requests are aggregated.
+
+        Returns:
+            Dict mapping metric name to dict of replica ID to aggregated metric value.
+        """
+        aggregated_metrics = defaultdict(dict)
+
+        for replica_id in self._running_replicas:
+            replica_metric_report = self._replica_metrics.get(replica_id)
+            if replica_metric_report is None:
+                continue
+
+            for metric_name, timeseries in replica_metric_report.metrics.items():
+                # Aggregate the timeseries for this custom metric
+                aggregated_value = self._merge_and_aggregate_timeseries([timeseries])
+                aggregated_metrics[metric_name][replica_id] = aggregated_value
+
+        return dict(aggregated_metrics)
+
+    def _get_raw_custom_metrics(
+        self,
+    ) -> Dict[str, Dict[ReplicaID, TimeSeries]]:
+        """Extract raw custom metric values from replica metric reports.
+
+        Returns:
+            Dict mapping metric name to dict of replica ID to raw metric timeseries.
+        """
+        raw_metrics = defaultdict(dict)
+
+        for replica_id in self._running_replicas:
+            replica_metric_report = self._replica_metrics.get(replica_id)
+            if replica_metric_report is None:
+                continue
+
+            for metric_name, timeseries in replica_metric_report.metrics.items():
+                # Extract values from TimeStampedValue list
+                raw_metrics[metric_name][replica_id] = timeseries
+
+        return dict(raw_metrics)
+
+
+class ApplicationAutoscalingState:
+    """Manages autoscaling for a single application."""
+
+    def __init__(
+        self,
+        app_name: ApplicationName,
+    ):
+        self._app_name = app_name
+        self._deployment_autoscaling_states: Dict[
+            DeploymentID, DeploymentAutoscalingState
+        ] = {}
+        self._policy: Optional[
+            Callable[
+                [Dict[DeploymentID, AutoscalingContext]],
+                Tuple[Dict[DeploymentID, int], Optional[Dict[str, Dict]]],
+            ]
+        ] = None
+        # user defined policy returns a dictionary of state that is persisted between autoscaling decisions
+        # content of the dictionary is determined by the user defined policy
+        self._policy_state: Optional[Dict[str, Any]] = None
+
+    @property
+    def deployments(self):
+        return self._deployment_autoscaling_states.keys()
+
+    def register(
+        self,
+        autoscaling_policy: AutoscalingPolicy,
+    ):
+        """Register or update application-level autoscaling config and deployments.
+
+        This will overwrite the deployment-level policies with the application-level policy.
+
+        Args:
+            autoscaling_policy: The autoscaling policy to register.
+        """
+        self._policy = autoscaling_policy.get_policy()
+        self._policy_state = {}
+
+        # Log when custom autoscaling policy is used for application
+        if not autoscaling_policy.is_default_policy_function():
+            logger.info(
+                f"Using custom autoscaling policy '{autoscaling_policy.policy_function}' "
+                f"for application '{self._app_name}'."
+            )
+            # Record telemetry for custom autoscaling policy usage
+            ServeUsageTag.CUSTOM_AUTOSCALING_POLICY_USED.record("1")
+
+    def has_policy(self) -> bool:
+        return self._policy is not None
+
+    def register_deployment(
+        self,
+        deployment_id: DeploymentID,
+        info: DeploymentInfo,
+        curr_target_num_replicas: int,
+    ) -> int:
+        """Register a single deployment under this application."""
+        if deployment_id not in self._deployment_autoscaling_states:
+            self._deployment_autoscaling_states[
+                deployment_id
+            ] = DeploymentAutoscalingState(deployment_id)
+
+        if info.deployment_config.autoscaling_config is None:
+            raise ValueError(
+                f"Autoscaling config is not set for deployment {deployment_id}"
+            )
+
+        # if the deployment-level policy is not the default policy, and the application has a policy,
+        # warn the user that the application-level policy will take precedence
+        if (
+            not info.deployment_config.autoscaling_config.policy.is_default_policy_function()
+            and self.has_policy()
+        ):
+            logger.warning(
+                f"User provided both a deployment-level and an application-level policy for deployment {deployment_id}. "
+                "The application-level policy will take precedence."
+            )
+
+        return self._deployment_autoscaling_states[deployment_id].register(
+            info,
+            curr_target_num_replicas,
+        )
+
+    def deregister_deployment(self, deployment_id: DeploymentID):
+        if deployment_id not in self._deployment_autoscaling_states:
+            logger.warning(
+                f"Cannot deregister autoscaling state for deployment {deployment_id} because it is not registered"
+            )
+            return
+        self._deployment_autoscaling_states.pop(deployment_id)
+
+    def should_autoscale_deployment(self, deployment_id: DeploymentID):
+        return deployment_id in self._deployment_autoscaling_states
+
+    def get_decision_num_replicas(
+        self,
+        deployment_to_target_num_replicas: Dict[DeploymentID, int],
+        _skip_bound_check: bool = False,
+    ) -> Dict[DeploymentID, int]:
+        """
+        Decide scaling for all deployments in this application by calling
+        each deployment's autoscaling policy.
+        """
+        if self.has_policy():
+            # Using app-level policy
+            autoscaling_contexts = {
+                deployment_id: state.get_autoscaling_context(
+                    deployment_to_target_num_replicas[deployment_id]
+                )
+                for deployment_id, state in self._deployment_autoscaling_states.items()
+            }
+
+            # Policy returns {deployment_name -> decision}
+            decisions, self._policy_state = self._policy(autoscaling_contexts)
+
+            assert (
+                type(decisions) is dict
+            ), "Autoscaling policy must return a dictionary of deployment_name -> decision_num_replicas"
+
+            # assert that deployment_id is in decisions is valid
+            for deployment_id in decisions.keys():
+                assert (
+                    deployment_id in self._deployment_autoscaling_states
+                ), f"Deployment {deployment_id} is not registered"
+                assert (
+                    deployment_id in deployment_to_target_num_replicas
+                ), f"Deployment {deployment_id} is invalid"
+
+            return {
+                deployment_id: (
+                    self._deployment_autoscaling_states[deployment_id].apply_bounds(
+                        num_replicas
+                    )
+                    if not _skip_bound_check
+                    else num_replicas
+                )
+                for deployment_id, num_replicas in decisions.items()
+            }
+        else:
+            # Using deployment-level policy
+            return {
+                deployment_id: deployment_autoscaling_state.get_decision_num_replicas(
+                    curr_target_num_replicas=deployment_to_target_num_replicas[
+                        deployment_id
+                    ],
+                    _skip_bound_check=_skip_bound_check,
+                )
+                for deployment_id, deployment_autoscaling_state in self._deployment_autoscaling_states.items()
+            }
+
+    def update_running_replica_ids(
+        self, deployment_id: DeploymentID, running_replicas: List[ReplicaID]
+    ):
+        self._deployment_autoscaling_states[deployment_id].update_running_replica_ids(
+            running_replicas
+        )
+
+    def on_replica_stopped(self, replica_id: ReplicaID):
+        dep_id = replica_id.deployment_id
+        if dep_id in self._deployment_autoscaling_states:
+            self._deployment_autoscaling_states[dep_id].on_replica_stopped(replica_id)
+
+    def get_total_num_requests_for_deployment(
+        self, deployment_id: DeploymentID
+    ) -> float:
+        return self._deployment_autoscaling_states[
+            deployment_id
+        ].get_total_num_requests()
+
+    def get_replica_metrics_by_deployment_id(self, deployment_id: DeploymentID):
+        return self._deployment_autoscaling_states[deployment_id].get_replica_metrics()
+
+    def is_within_bounds(
+        self, deployment_id: DeploymentID, num_replicas_running_at_target_version: int
+    ) -> bool:
+        return self._deployment_autoscaling_states[deployment_id].is_within_bounds(
+            num_replicas_running_at_target_version
+        )
+
+    def record_request_metrics_for_replica(
+        self, replica_metric_report: ReplicaMetricReport
+    ):
+        dep_id = replica_metric_report.replica_id.deployment_id
+        # Defensively guard against delayed replica metrics arriving
+        # after the deployment's been deleted
+        if dep_id in self._deployment_autoscaling_states:
+            self._deployment_autoscaling_states[
+                dep_id
+            ].record_request_metrics_for_replica(replica_metric_report)
+
+    def record_request_metrics_for_handle(
+        self, handle_metric_report: HandleMetricReport
+    ):
+        dep_id = handle_metric_report.deployment_id
+        if dep_id in self._deployment_autoscaling_states:
+            self._deployment_autoscaling_states[
+                dep_id
+            ].record_request_metrics_for_handle(handle_metric_report)
+
+    def drop_stale_handle_metrics(self, alive_serve_actor_ids: Set[str]):
+        """Drops handle metrics that are no longer valid.
+
+        This includes handles that live on Serve Proxy or replica actors
+        that have died AND handles from which the controller hasn't
+        received an update for too long.
+        """
+        for dep_state in self._deployment_autoscaling_states.values():
+            dep_state.drop_stale_handle_metrics(alive_serve_actor_ids)
+
 
 class AutoscalingStateManager:
     """Manages all things autoscaling related.
 
-    Keeps track of request metrics for each deployment and decides on
-    the target number of replicas to autoscale to based on those metrics.
+    Keeps track of request metrics for each application and its deployments,
+    and decides on the target number of replicas to autoscale to.
     """
 
     def __init__(self):
-        self._autoscaling_states: Dict[DeploymentID, AutoscalingState] = {}
+        self._app_autoscaling_states: Dict[
+            ApplicationName, ApplicationAutoscalingState
+        ] = {}
 
     def register_deployment(
         self,
@@ -610,91 +874,137 @@ class AutoscalingStateManager:
     ) -> int:
         """Register autoscaling deployment info."""
         assert info.deployment_config.autoscaling_config
-        if deployment_id not in self._autoscaling_states:
-            self._autoscaling_states[deployment_id] = AutoscalingState(deployment_id)
-        return self._autoscaling_states[deployment_id].register(
-            info, curr_target_num_replicas
+        app_name = deployment_id.app_name
+        app_state = self._app_autoscaling_states.setdefault(
+            app_name, ApplicationAutoscalingState(app_name)
+        )
+        logger.info(f"Registering autoscaling state for deployment {deployment_id}")
+        return app_state.register_deployment(
+            deployment_id, info, curr_target_num_replicas
         )
 
     def deregister_deployment(self, deployment_id: DeploymentID):
         """Remove deployment from tracking."""
-        self._autoscaling_states.pop(deployment_id, None)
+        app_state = self._app_autoscaling_states.get(deployment_id.app_name)
+        if app_state:
+            logger.info(
+                f"Deregistering autoscaling state for deployment {deployment_id}"
+            )
+            app_state.deregister_deployment(deployment_id)
+
+    def register_application(
+        self,
+        app_name: ApplicationName,
+        autoscaling_policy: AutoscalingPolicy,
+    ):
+        app_state = self._app_autoscaling_states.setdefault(
+            app_name, ApplicationAutoscalingState(app_name)
+        )
+        logger.info(f"Registering autoscaling state for application {app_name}")
+        app_state.register(autoscaling_policy)
+
+    def deregister_application(self, app_name: ApplicationName):
+        """Remove application from tracking."""
+        if app_name in self._app_autoscaling_states:
+            logger.info(f"Deregistering autoscaling state for application {app_name}")
+            self._app_autoscaling_states.pop(app_name, None)
+
+    def _application_has_policy(self, app_name: ApplicationName) -> bool:
+        return (
+            app_name in self._app_autoscaling_states
+            and self._app_autoscaling_states[app_name].has_policy()
+        )
+
+    def get_decision_num_replicas(
+        self,
+        app_name: ApplicationName,
+        deployment_to_target_num_replicas: Dict[DeploymentID, int],
+    ) -> Dict[DeploymentID, int]:
+        """
+        Decide scaling for all deployments in the application.
+
+        Args:
+            app_name: The name of the application.
+            deployment_to_target_num_replicas: A dictionary of deployment_id to target number of replicas.
+
+        Returns:
+            A dictionary of deployment_id to decision number of replicas.
+        """
+        return self._app_autoscaling_states[app_name].get_decision_num_replicas(
+            deployment_to_target_num_replicas
+        )
+
+    def should_autoscale_application(self, app_name: ApplicationName):
+        return app_name in self._app_autoscaling_states
+
+    def should_autoscale_deployment(self, deployment_id: DeploymentID):
+        return (
+            deployment_id.app_name in self._app_autoscaling_states
+            and self._app_autoscaling_states[
+                deployment_id.app_name
+            ].should_autoscale_deployment(deployment_id)
+        )
 
     def update_running_replica_ids(
         self, deployment_id: DeploymentID, running_replicas: List[ReplicaID]
     ):
-        self._autoscaling_states[deployment_id].update_running_replica_ids(
-            running_replicas
-        )
+        app_state = self._app_autoscaling_states.get(deployment_id.app_name)
+        if app_state:
+            app_state.update_running_replica_ids(deployment_id, running_replicas)
 
     def on_replica_stopped(self, replica_id: ReplicaID):
-        deployment_id = replica_id.deployment_id
-        if deployment_id in self._autoscaling_states:
-            self._autoscaling_states[deployment_id].on_replica_stopped(replica_id)
+        app_state = self._app_autoscaling_states.get(replica_id.deployment_id.app_name)
+        if app_state:
+            app_state.on_replica_stopped(replica_id)
 
-    def get_metrics(self) -> Dict[DeploymentID, float]:
-        return {
-            deployment_id: self.get_total_num_requests(deployment_id)
-            for deployment_id in self._autoscaling_states
-        }
+    def get_metrics_for_deployment(
+        self, deployment_id: DeploymentID
+    ) -> Dict[ReplicaID, List[TimeSeries]]:
+        if deployment_id.app_name in self._app_autoscaling_states:
+            return self._app_autoscaling_states[
+                deployment_id.app_name
+            ].get_replica_metrics_by_deployment_id(deployment_id)
+        else:
+            return {}
 
-    def get_all_metrics(
-        self, agg_func="mean"
-    ) -> Dict[DeploymentID, Dict[ReplicaID, List[Any]]]:
-        return {
-            deployment_id: self._autoscaling_states[deployment_id].get_replica_metrics(
-                agg_func
-            )
-            for deployment_id in self._autoscaling_states
-        }
-
-    def get_target_num_replicas(
-        self, deployment_id: DeploymentID, curr_target_num_replicas: int
-    ) -> int:
-        return self._autoscaling_states[deployment_id].get_decision_num_replicas(
-            curr_target_num_replicas=curr_target_num_replicas,
-        )
-
-    def get_total_num_requests(self, deployment_id: DeploymentID) -> float:
-        return self._autoscaling_states[deployment_id].get_total_num_requests()
+    def get_total_num_requests_for_deployment(
+        self, deployment_id: DeploymentID
+    ) -> float:
+        if deployment_id.app_name in self._app_autoscaling_states:
+            return self._app_autoscaling_states[
+                deployment_id.app_name
+            ].get_total_num_requests_for_deployment(deployment_id)
+        else:
+            return 0
 
     def is_within_bounds(
         self, deployment_id: DeploymentID, num_replicas_running_at_target_version: int
     ) -> bool:
-        return self._autoscaling_states[deployment_id].is_within_bounds(
-            num_replicas_running_at_target_version
+        app_state = self._app_autoscaling_states[deployment_id.app_name]
+        return app_state.is_within_bounds(
+            deployment_id, num_replicas_running_at_target_version
         )
 
     def record_request_metrics_for_replica(
         self, replica_metric_report: ReplicaMetricReport
     ) -> None:
-        deployment_id = replica_metric_report.replica_id.deployment_id
-        # Defensively guard against delayed replica metrics arriving
-        # after the deployment's been deleted
-        if deployment_id in self._autoscaling_states:
-            self._autoscaling_states[deployment_id].record_request_metrics_for_replica(
-                replica_metric_report
-            )
+        app_state = self._app_autoscaling_states.get(
+            replica_metric_report.replica_id.deployment_id.app_name
+        )
+        if app_state:
+            app_state.record_request_metrics_for_replica(replica_metric_report)
 
     def record_request_metrics_for_handle(
         self,
         handle_metric_report: HandleMetricReport,
     ) -> None:
         """Update request metric for a specific handle."""
-
-        deployment_id = handle_metric_report.deployment_id
-        if deployment_id in self._autoscaling_states:
-            self._autoscaling_states[deployment_id].record_request_metrics_for_handle(
-                handle_metric_report
-            )
+        app_state = self._app_autoscaling_states.get(
+            handle_metric_report.deployment_id.app_name
+        )
+        if app_state:
+            app_state.record_request_metrics_for_handle(handle_metric_report)
 
     def drop_stale_handle_metrics(self, alive_serve_actor_ids: Set[str]) -> None:
-        """Drops handle metrics that are no longer valid.
-
-        This includes handles that live on Serve Proxy or replica actors
-        that have died AND handles from which the controller hasn't
-        received an update for too long.
-        """
-
-        for autoscaling_state in self._autoscaling_states.values():
-            autoscaling_state.drop_stale_handle_metrics(alive_serve_actor_ids)
+        for app_state in self._app_autoscaling_states.values():
+            app_state.drop_stale_handle_metrics(alive_serve_actor_ids)
