@@ -1,3 +1,5 @@
+import asyncio
+import inspect
 import logging
 import random
 import time
@@ -18,6 +20,7 @@ from ray.serve._private.common import (
 from ray.serve._private.constants import (
     CLIENT_CHECK_CREATION_POLLING_INTERVAL_S,
     CLIENT_POLLING_INTERVAL_S,
+    HTTP_PROXY_TIMEOUT,
     MAX_CACHED_HANDLES,
     SERVE_DEFAULT_APP_NAME,
     SERVE_LOGGER_NAME,
@@ -25,6 +28,7 @@ from ray.serve._private.constants import (
 from ray.serve._private.controller import ServeController
 from ray.serve._private.deploy_utils import get_deploy_args
 from ray.serve._private.deployment_info import DeploymentInfo
+from ray.serve._private.http_util import ASGIAppReplicaWrapper
 from ray.serve._private.utils import get_random_string
 from ray.serve.config import HTTPOptions
 from ray.serve.exceptions import RayServeException
@@ -80,18 +84,31 @@ class ServeControllerClient:
     def __reduce__(self):
         raise RayServeException(("Ray Serve client cannot be serialized."))
 
-    def shutdown_cached_handles(self, _skip_asyncio_check: bool = False):
+    def shutdown_cached_handles(self):
         """Shuts down all cached handles.
 
         Remove the reference to the cached handles so that they can be
         garbage collected.
         """
         for cache_key in list(self.handle_cache):
-            self.handle_cache[cache_key].shutdown(
-                _skip_asyncio_check=_skip_asyncio_check
-            )
+            self.handle_cache[cache_key].shutdown()
             del self.handle_cache[cache_key]
 
+    async def shutdown_cached_handles_async(self):
+        """Shuts down all cached handles asynchronously.
+
+        Remove the reference to the cached handles so that they can be
+        garbage collected.
+        """
+
+        async def shutdown_task(cache_key):
+            await self.handle_cache[cache_key].shutdown_async()
+            del self.handle_cache[cache_key]
+
+        await asyncio.gather(
+            *[shutdown_task(cache_key) for cache_key in list(self.handle_cache)]
+        )
+
     def shutdown(self, timeout_s: float = 30.0) -> None:
         """Completely shut down the connected Serve instance.
 
@@ -113,6 +130,29 @@ class ServeControllerClient:
                 )
             self._shutdown = True
 
+    async def shutdown_async(self, timeout_s: float = 30.0) -> None:
+        """Completely shut down the connected Serve instance.
+
+        Shuts down all processes and deletes all state associated with the
+        instance.
+        """
+        await self.shutdown_cached_handles_async()
+
+        if ray.is_initialized() and not self._shutdown:
+            try:
+                await asyncio.wait_for(
+                    self._controller.graceful_shutdown.remote(), timeout=timeout_s
+                )
+            except ray.exceptions.RayActorError:
+                # Controller has been shut down.
+                pass
+            except TimeoutError:
+                logger.warning(
+                    f"Controller failed to shut down within {timeout_s}s. "
+                    "Check controller logs for more details."
+                )
+            self._shutdown = True
+
     def _wait_for_deployment_healthy(self, name: str, timeout_s: int = -1):
         """Waits for the named deployment to enter "HEALTHY" status.
 
@@ -249,6 +289,39 @@ class ServeControllerClient:
                 f"Application {name} did not become RUNNING after {timeout_s}s."
             )
 
+    @_ensure_connected
+    def wait_for_proxies_serving(
+        self, wait_for_applications_running: bool = True
+    ) -> None:
+        """Wait for the proxies to be ready to serve requests."""
+        proxy_handles = ray.get(self._controller.get_proxies.remote())
+        serving_refs = [
+            handle.serving.remote(
+                wait_for_applications_running=wait_for_applications_running
+            )
+            for handle in proxy_handles.values()
+        ]
+
+        done, pending = ray.wait(
+            serving_refs,
+            timeout=HTTP_PROXY_TIMEOUT,
+            num_returns=len(serving_refs),
+        )
+
+        if len(pending) > 0:
+            raise TimeoutError(f"Proxies not available after {HTTP_PROXY_TIMEOUT}s.")
+
+        # Ensure the proxies are either serving or dead.
+        for ref in done:
+            try:
+                ray.get(ref, timeout=1)
+            except ray.exceptions.RayActorError:
+                pass
+            except Exception:
+                raise TimeoutError(
+                    f"Proxies not available after {HTTP_PROXY_TIMEOUT}s."
+                )
+
     @_ensure_connected
     def deploy_applications(
         self,
@@ -295,6 +368,9 @@ class ServeControllerClient:
 
             name_to_deployment_args_list[app.name] = deployment_args_list
 
+        # Validate applications before sending to controller
+        self._check_ingress_deployments(built_apps)
+
         ray.get(
             self._controller.deploy_applications.remote(name_to_deployment_args_list)
         )
@@ -346,17 +422,55 @@ class ServeControllerClient:
         if _blocking:
             timeout_s = 60
 
+            if isinstance(config, ServeDeploySchema):
+                app_names = {app.name for app in config.applications}
+            else:
+                app_names = {config.name}
+
             start = time.time()
             while time.time() - start < timeout_s:
-                curr_status = self.get_serve_status()
-                if curr_status.app_status.status == ApplicationStatus.RUNNING:
+                statuses = self.list_serve_statuses()
+                app_to_status = {
+                    status.name: status.app_status.status
+                    for status in statuses
+                    if status.name in app_names
+                }
+                if len(app_names) == len(app_to_status) and set(
+                    app_to_status.values()
+                ) == {ApplicationStatus.RUNNING}:
                     break
+
                 time.sleep(CLIENT_POLLING_INTERVAL_S)
             else:
                 raise TimeoutError(
                     f"Serve application isn't running after {timeout_s}s."
                 )
 
+            self.wait_for_proxies_serving(wait_for_applications_running=True)
+
+    def _check_ingress_deployments(
+        self, built_apps: Sequence[BuiltApplication]
+    ) -> None:
+        """Check @serve.ingress of deployments across applications.
+
+        Raises: RayServeException if more than one @serve.ingress
+            is found among deployments in any single application.
+        """
+        for app in built_apps:
+            num_ingress_deployments = 0
+            for deployment in app.deployments:
+                if inspect.isclass(deployment.func_or_class) and issubclass(
+                    deployment.func_or_class, ASGIAppReplicaWrapper
+                ):
+                    num_ingress_deployments += 1
+
+            if num_ingress_deployments > 1:
+                raise RayServeException(
+                    f'Found multiple FastAPI deployments in application "{app.name}".'
+                    "Please only include one deployment with @serve.ingress "
+                    "in your application to avoid this issue."
+                )
+
     @_ensure_connected
     def delete_apps(self, names: List[str], blocking: bool = True):
         if not names:
@@ -414,6 +528,14 @@ class ServeControllerClient:
         )
         return StatusOverview.from_proto(proto)
 
+    @_ensure_connected
+    def list_serve_statuses(self) -> List[StatusOverview]:
+        statuses_bytes = ray.get(self._controller.list_serve_statuses.remote())
+        return [
+            StatusOverview.from_proto(StatusOverviewProto.FromString(status_bytes))
+            for status_bytes in statuses_bytes
+        ]
+
     @_ensure_connected
     def get_all_deployment_statuses(self) -> List[DeploymentStatusInfo]:
         statuses_bytes = ray.get(self._controller.get_all_deployment_statuses.remote())
