@@ -501,10 +501,16 @@ HEALTHY_MESSAGE = "success"
 # This should be at the end.
 RAY_SERVE_THROUGHPUT_OPTIMIZED = get_env_bool("RAY_SERVE_THROUGHPUT_OPTIMIZED", "0")
 if RAY_SERVE_THROUGHPUT_OPTIMIZED:
-    RAY_SERVE_RUN_USER_CODE_IN_SEPARATE_THREAD = False
-    RAY_SERVE_REQUEST_PATH_LOG_BUFFER_SIZE = 1000
-    RAY_SERVE_RUN_ROUTER_IN_SEPARATE_LOOP = False
-    RAY_SERVE_LOG_TO_STDERR = False
+    RAY_SERVE_RUN_USER_CODE_IN_SEPARATE_THREAD = get_env_bool(
+        "RAY_SERVE_RUN_USER_CODE_IN_SEPARATE_THREAD", "0"
+    )
+    RAY_SERVE_REQUEST_PATH_LOG_BUFFER_SIZE = get_env_int(
+        "RAY_SERVE_REQUEST_PATH_LOG_BUFFER_SIZE", 1000
+    )
+    RAY_SERVE_RUN_ROUTER_IN_SEPARATE_LOOP = get_env_bool(
+        "RAY_SERVE_RUN_ROUTER_IN_SEPARATE_LOOP", "0"
+    )
+    RAY_SERVE_LOG_TO_STDERR = get_env_bool("RAY_SERVE_LOG_TO_STDERR", "0")
 
 # The maximum allowed RPC latency in milliseconds.
 # This is used to detect and warn about long RPC latencies
