@@ -20,7 +20,9 @@ from typing import (
     Callable,
     Dict,
     Generator,
+    List,
     Optional,
+    Set,
     Tuple,
     Union,
 )
@@ -36,6 +38,7 @@ from ray import cloudpickle
 from ray._common.filters import CoreContextFilter
 from ray._common.utils import get_or_create_event_loop
 from ray.actor import ActorClass, ActorHandle
+from ray.dag.py_obj_scanner import _PyObjScanner
 from ray.remote_function import RemoteFunction
 from ray.serve import metrics
 from ray.serve._private.common import (
@@ -93,7 +96,10 @@ from ray.serve._private.logging_utils import (
 )
 from ray.serve._private.metrics_utils import InMemoryMetricsStore, MetricsPusher
 from ray.serve._private.task_consumer import TaskConsumerWrapper
-from ray.serve._private.thirdparty.get_asgi_route_name import get_asgi_route_name
+from ray.serve._private.thirdparty.get_asgi_route_name import (
+    extract_route_patterns,
+    get_asgi_route_name,
+)
 from ray.serve._private.usage import ServeUsageTag
 from ray.serve._private.utils import (
     Semaphore,
@@ -109,7 +115,8 @@ from ray.serve.exceptions import (
     DeploymentUnavailableError,
     RayServeException,
 )
-from ray.serve.schema import EncodingType, LoggingConfig
+from ray.serve.handle import DeploymentHandle
+from ray.serve.schema import EncodingType, LoggingConfig, ReplicaRank
 
 logger = logging.getLogger(SERVE_LOGGER_NAME)
 
@@ -121,6 +128,10 @@ ReplicaMetadata = Tuple[
     Optional[int],
     Optional[str],
     int,
+    int,
+    ReplicaRank,  # rank
+    Optional[List[str]],  # route_patterns
+    Optional[List[DeploymentID]],  # outbound_deployments
 ]
 
 
@@ -304,35 +315,35 @@ class ReplicaMetricsManager:
     def should_collect_ongoing_requests(self) -> bool:
         """Determine if replicas should collect ongoing request metrics.
 
-        ┌─────────────────────────────────────────────────────────────────┐
-        │  Replica-based metrics collection                               │
-        ├─────────────────────────────────────────────────────────────────┤
-        │                                                                 │
-        │  Client          Handle            Replicas                     │
-        │  ┌──────┐      ┌────────┐                                       │
-        │  │  App │─────>│ Handle │────┬───>┌─────────┐                  │
-        │  │      │      │ Tracks │    │    │ Replica │                  │
-        │  └──────┘      │ Queued │    │    │    1    │                  │
-        │                │Requests│    │    │ Tracks  │                  │
-        │                └────────┘    │    │ Running │                  │
-        │                     │        │    └─────────┘                  │
-        │                     │        │         │                       │
-        │                     │        │         │                       │
-        │                     │        │    ┌─────────┐                  │
-        │                     │        └───>│ Replica │                  │
-        │                     │             │    2    │                  │
-        │                     │             │ Tracks  │                  │
-        │                     │             │ Running │                  │
-        │                     │             └─────────┘                  │
-        │                     │                  │                        │
-        │                     │                  │                        │
-        │                     ▼                  ▼                        │
-        │              ┌──────────────────────────────┐                  │
-        │              │        Controller            │                  │
-        │              │  • Queued metrics (handle)   │                  │
-        │              │  • Running metrics (replica1)│                  │
-        │              │  • Running metrics (replica2)│                  │
-        │              └──────────────────────────────┘                  │
+        ┌────────────────────────────────────────────────────────────────┐
+        │  Replica-based metrics collection                              │
+        ├────────────────────────────────────────────────────────────────┤
+        │                                                                │
+        │      Client          Handle            Replicas                │
+        │      ┌──────┐      ┌────────┐                                  │
+        │      │  App │─────>│ Handle │────┬───>┌─────────┐              │
+        │      │      │      │ Tracks │    │    │ Replica │              │
+        │      └──────┘      │ Queued │    │    │    1    │              │
+        │                    │Requests│    │    │ Tracks  │              │
+        │                    └────────┘    │    │ Running │              │
+        │                         │        │    └─────────┘              │
+        │                         │        │         │                   │
+        │                         │        │         │                   │
+        │                         │        │    ┌─────────┐              │
+        │                         │        └───>│ Replica │              │
+        │                         │             │    2    │              │
+        │                         │             │ Tracks  │              │
+        │                         │             │ Running │              │
+        │                         │             └─────────┘              │
+        │                         │                  │                   │
+        │                         │                  │                   │
+        │                         ▼                  ▼                   │
+        │                  ┌──────────────────────────────┐              │
+        │                  │        Controller            │              │
+        │                  │  • Queued metrics (handle)   │              │
+        │                  │  • Running metrics (replica1)│              │
+        │                  │  • Running metrics (replica2)│              │
+        │                  └──────────────────────────────┘              │
         │                                                                │
         └────────────────────────────────────────────────────────────────┘
         """
@@ -500,7 +511,7 @@ class ReplicaBase(ABC):
         version: DeploymentVersion,
         ingress: bool,
         route_prefix: str,
-        rank: int,
+        rank: ReplicaRank,
     ):
         self._version = version
         self._replica_id = replica_id
@@ -535,6 +546,9 @@ class ReplicaBase(ABC):
         self._user_callable_initialized_lock = asyncio.Lock()
         self._initialization_latency: Optional[float] = None
 
+        # Track deployment handles created dynamically via get_deployment_handle()
+        self._dynamically_created_handles: Set[DeploymentID] = set()
+
         # Flipped to `True` when health checks pass and `False` when they fail. May be
         # used by replica subclass implementations.
         self._healthy = False
@@ -573,6 +587,14 @@ class ReplicaBase(ABC):
 
     def get_metadata(self) -> ReplicaMetadata:
         current_rank = ray.serve.context._get_internal_replica_context().rank
+        # Extract route patterns from ASGI app if available
+        route_patterns = None
+        if self._user_callable_asgi_app is not None:
+            # _user_callable_asgi_app is the actual ASGI app (FastAPI/Starlette)
+            # It's set when initialize_callable() returns an ASGI app
+            if hasattr(self._user_callable_asgi_app, "routes"):
+                route_patterns = extract_route_patterns(self._user_callable_asgi_app)
+
         return (
             self._version.deployment_config,
             self._version,
@@ -582,19 +604,69 @@ class ReplicaBase(ABC):
             self._http_port,
             self._grpc_port,
             current_rank,
+            route_patterns,
+            self.list_outbound_deployments(),
         )
 
+    def get_dynamically_created_handles(self) -> Set[DeploymentID]:
+        return self._dynamically_created_handles
+
+    def list_outbound_deployments(self) -> List[DeploymentID]:
+        """List all outbound deployment IDs this replica calls into.
+
+        This includes:
+        - Handles created via get_deployment_handle()
+        - Handles passed as init args/kwargs to the deployment constructor
+
+        This is used to determine which deployments are reachable from this replica.
+        The list of DeploymentIDs can change over time as new handles can be created at runtime.
+        Also its not guaranteed that the list of DeploymentIDs are identical across replicas
+        because it depends on user code.
+
+        Returns:
+            A list of DeploymentIDs that this replica calls into.
+        """
+        seen_deployment_ids: Set[DeploymentID] = set()
+
+        # First, collect dynamically created handles
+        for deployment_id in self.get_dynamically_created_handles():
+            seen_deployment_ids.add(deployment_id)
+
+        # Get the init args/kwargs
+        init_args = self._user_callable_wrapper._init_args
+        init_kwargs = self._user_callable_wrapper._init_kwargs
+
+        # Use _PyObjScanner to find all DeploymentHandle objects in:
+        # The init_args and init_kwargs (handles might be passed as init args)
+        scanner = _PyObjScanner(source_type=DeploymentHandle)
+        try:
+            handles = scanner.find_nodes((init_args, init_kwargs))
+
+            for handle in handles:
+                deployment_id = handle.deployment_id
+                seen_deployment_ids.add(deployment_id)
+        finally:
+            scanner.clear()
+
+        return list(seen_deployment_ids)
+
     def _set_internal_replica_context(
-        self, *, servable_object: Callable = None, rank: int = None
+        self, *, servable_object: Callable = None, rank: ReplicaRank = None
     ):
         # Calculate world_size from deployment config instead of storing it
         world_size = self._deployment_config.num_replicas
+
+        # Create callback for registering dynamically created handles
+        def register_handle_callback(deployment_id: DeploymentID) -> None:
+            self._dynamically_created_handles.add(deployment_id)
+
         ray.serve.context._set_internal_replica_context(
             replica_id=self._replica_id,
             servable_object=servable_object,
             _deployment_config=self._deployment_config,
             rank=rank,
             world_size=world_size,
+            handle_registration_callback=register_handle_callback,
         )
 
     def _configure_logger_and_profilers(
@@ -930,7 +1002,7 @@ class ReplicaBase(ABC):
     async def reconfigure(
         self,
         deployment_config: DeploymentConfig,
-        rank: int,
+        rank: ReplicaRank,
         route_prefix: Optional[str] = None,
     ):
         try:
@@ -1155,7 +1227,7 @@ class ReplicaActor:
         version: DeploymentVersion,
         ingress: bool,
         route_prefix: str,
-        rank: int,
+        rank: ReplicaRank,
     ):
         deployment_config = DeploymentConfig.from_proto_bytes(
             deployment_config_proto_bytes
@@ -1211,6 +1283,9 @@ class ReplicaActor:
             get_component_logger_file_path(),
         )
 
+    def list_outbound_deployments(self) -> Optional[List[DeploymentID]]:
+        return self._replica_impl.list_outbound_deployments()
+
     async def initialize_and_get_metadata(
         self, deployment_config: DeploymentConfig = None, _after: Optional[Any] = None
     ) -> ReplicaMetadata:
@@ -1235,7 +1310,7 @@ class ReplicaActor:
         return await self._replica_impl.record_routing_stats()
 
     async def reconfigure(
-        self, deployment_config, rank: int, route_prefix: Optional[str] = None
+        self, deployment_config, rank: ReplicaRank, route_prefix: Optional[str] = None
     ) -> ReplicaMetadata:
         await self._replica_impl.reconfigure(deployment_config, rank, route_prefix)
         return self._replica_impl.get_metadata()
@@ -1732,7 +1807,7 @@ class UserCallableWrapper:
         return result
 
     @_run_user_code
-    async def call_reconfigure(self, user_config: Optional[Any], rank: int):
+    async def call_reconfigure(self, user_config: Optional[Any], rank: ReplicaRank):
         self._raise_if_not_initialized("call_reconfigure")
 
         # NOTE(edoakes): there is the possibility of a race condition in user code if
