@@ -1,6 +1,5 @@
 import asyncio
 import collections
-import concurrent.futures
 import copy
 import importlib
 import inspect
@@ -10,7 +9,6 @@ import re
 import time
 import uuid
 from abc import ABC, abstractmethod
-from asyncio import coroutines, ensure_future, futures
 from decimal import ROUND_HALF_UP, Decimal
 from enum import Enum
 from functools import wraps
@@ -27,7 +25,6 @@ from ray._raylet import MessagePackSerializer
 from ray.actor import ActorHandle
 from ray.serve._private.common import RequestMetadata, ServeComponentType
 from ray.serve._private.constants import HTTP_PROXY_TIMEOUT, SERVE_LOGGER_NAME
-from ray.serve.config import gRPCOptions
 from ray.types import ObjectRef
 from ray.util.serialization import StandaloneSerializationContext
 
@@ -44,6 +41,27 @@ except ImportError:
 FILE_NAME_REGEX = r"[^\x20-\x7E]|[<>:\"/\\|?*]"
 
 MESSAGE_PACK_OFFSET = 9
+
+
+def validate_ssl_config(
+    ssl_certfile: Optional[str], ssl_keyfile: Optional[str]
+) -> None:
+    """Validate SSL configuration for HTTPS support.
+
+    Args:
+        ssl_certfile: Path to SSL certificate file
+        ssl_keyfile: Path to SSL private key file
+
+    Raises:
+        ValueError: If only one of ssl_certfile or ssl_keyfile is provided
+    """
+    if (ssl_certfile and not ssl_keyfile) or (ssl_keyfile and not ssl_certfile):
+        raise ValueError(
+            "Both ssl_keyfile and ssl_certfile must be provided together "
+            "to enable HTTPS."
+        )
+
+
 GENERATOR_COMPOSITION_NOT_SUPPORTED_ERROR = RuntimeError(
     "Streaming deployment handle results cannot be passed to "
     "downstream handle calls. If you have a use case requiring "
@@ -612,41 +630,10 @@ def wait_for_interrupt() -> None:
         raise
 
 
-def is_grpc_enabled(grpc_config: gRPCOptions) -> bool:
+def is_grpc_enabled(grpc_config) -> bool:
     return grpc_config.port > 0 and len(grpc_config.grpc_servicer_functions) > 0
 
 
-def run_coroutine_or_future_threadsafe(coro_or_future, loop):
-    """Submit a coroutine object or future to a given event loop.
-
-    Ref: https://github.com/python/cpython/blob/eef49c359505eaf109d519d39e53dfd3c78d066a/Lib/asyncio/tasks.py#L991
-
-    Return a concurrent.futures.Future to access the result.
-    """
-    if not coroutines.iscoroutine(coro_or_future) and not futures.isfuture(
-        coro_or_future
-    ):
-        raise TypeError("A coroutine object or future is required")
-
-    if futures.isfuture(coro_or_future):
-        assert loop == coro_or_future.get_loop()
-
-    future = concurrent.futures.Future()
-
-    def callback():
-        try:
-            futures._chain_future(ensure_future(coro_or_future, loop=loop), future)
-        except (SystemExit, KeyboardInterrupt):
-            raise
-        except BaseException as exc:
-            if future.set_running_or_notify_cancel():
-                future.set_exception(exc)
-            raise
-
-    loop.call_soon_threadsafe(callback)
-    return future
-
-
 class Semaphore:
     """Based on asyncio.Semaphore.
 
