@@ -1,4 +1,5 @@
 import logging
+from abc import ABC, abstractmethod
 from collections import Counter
 from dataclasses import dataclass, field
 from enum import Enum
@@ -25,6 +26,7 @@ from ray.serve._private.common import (
     ServeDeployMode,
 )
 from ray.serve._private.constants import (
+    DEFAULT_CONSUMER_CONCURRENCY,
     DEFAULT_GRPC_PORT,
     DEFAULT_MAX_ONGOING_REQUESTS,
     DEFAULT_UVICORN_KEEP_ALIVE_TIMEOUT_S,
@@ -560,6 +562,15 @@ class ServeApplicationSchema(BaseModel):
         default=[],
         description="Deployment options that override options specified in the code.",
     )
+    autoscaling_policy: Optional[dict] = Field(
+        default=None,
+        description=(
+            "Application-level autoscaling policy. "
+            "If null, serve fallbacks to autoscaling policy in each deployment. "
+            "This option is under development and not yet supported."
+        ),
+    )
+
     args: Dict = Field(
         default={},
         description="Arguments that will be passed to the application builder.",
@@ -1306,6 +1317,16 @@ class CeleryAdapterConfig(BaseModel):
     Celery adapter config. You can use it to configure the Celery task processor for your Serve application.
     """
 
+    app_custom_config: Optional[Dict[str, Any]] = Field(
+        default=None, description="The custom configurations to use for the Celery app."
+    )
+    task_custom_config: Optional[Dict[str, Any]] = Field(
+        default=None,
+        description="""
+        The custom configurations to use for the Celery task.
+        This custom configurations will get applied to all the celery tasks.
+        """,
+    )
     broker_url: str = Field(..., description="The URL of the broker to use for Celery.")
     backend_url: str = Field(
         ..., description="The URL of the backend to use for Celery."
@@ -1313,10 +1334,6 @@ class CeleryAdapterConfig(BaseModel):
     broker_transport_options: Optional[Dict[str, Any]] = Field(
         default=None, description="The broker transport options to use for Celery."
     )
-    worker_concurrency: Optional[int] = Field(
-        default=10,
-        description="The number of concurrent worker threads for the task processor.",
-    )
 
 
 @PublicAPI(stability="alpha")
@@ -1361,6 +1378,212 @@ class TaskResult(BaseModel):
     result: Any = Field(..., description="The result of the task.")
 
 
+@PublicAPI(stability="alpha")
+class TaskProcessorAdapter(ABC):
+    """
+    Abstract base class for task processing adapters.
+
+    Subclasses can support different combinations of sync and async operations.
+    Use supports_async_capability() to check if a specific async operation is supported.
+    """
+
+    def __init__(self, *args, **kwargs):
+        """
+        Initialize the TaskProcessorAdapter.
+
+        """
+        pass
+
+    @abstractmethod
+    def initialize(self, consumer_concurrency: int = DEFAULT_CONSUMER_CONCURRENCY):
+        """
+        Initialize the task processor.
+        """
+        pass
+
+    @abstractmethod
+    def register_task_handle(self, func: Callable, name: Optional[str] = None):
+        """
+        Register a function as a task handler.
+
+        Args:
+            func: The function to register as a task handler.
+            name: Custom name for the task.
+        """
+        pass
+
+    @abstractmethod
+    def enqueue_task_sync(
+        self,
+        task_name: str,
+        args: Optional[Any] = None,
+        kwargs: Optional[Any] = None,
+        **options,
+    ) -> TaskResult:
+        """
+        Enqueue a task for execution synchronously.
+
+        Args:
+            task_name: Name of the registered task to execute.
+            args: Positional arguments to pass to the task function.
+            kwargs: Keyword arguments to pass to the task function.
+            **options: Additional adapter-specific options for task execution.
+
+        Returns:
+            TaskResult: Object containing task ID, status, and other metadata.
+        """
+        pass
+
+    @abstractmethod
+    def get_task_status_sync(self, task_id: str) -> TaskResult:
+        """
+        Retrieve the current status of a task synchronously.
+
+        Args:
+            task_id: Unique identifier of the task to query.
+
+        Returns:
+            TaskResult: Object containing current task status, result, and other metadata.
+        """
+        pass
+
+    @abstractmethod
+    def start_consumer(self, **kwargs):
+        """
+        Start the task consumer/worker process.
+        """
+        pass
+
+    @abstractmethod
+    def stop_consumer(self, timeout: float = 10.0):
+        """
+        Stop the task consumer gracefully.
+
+        Args:
+            timeout: Maximum time in seconds to wait for the consumer to stop.
+        """
+        pass
+
+    @abstractmethod
+    def shutdown(self):
+        """
+        Shutdown the task processor and clean up resources.
+        """
+        pass
+
+    @abstractmethod
+    def cancel_task_sync(self, task_id: str):
+        """
+        Cancel a task synchronously.
+
+        Args:
+            task_id: Unique identifier of the task to cancel.
+        """
+        pass
+
+    @abstractmethod
+    def get_metrics_sync(self) -> Dict[str, Any]:
+        """
+        Get metrics synchronously.
+
+        Returns:
+            Dict[str, Any]: Adapter-specific metrics data.
+        """
+        pass
+
+    @abstractmethod
+    def health_check_sync(self) -> List[Dict]:
+        """
+        Perform health check synchronously.
+
+        Returns:
+            List[Dict]: Health status information for workers/components.
+        """
+        pass
+
+    async def enqueue_task_async(
+        self,
+        task_name: str,
+        args: Optional[Any] = None,
+        kwargs: Optional[Any] = None,
+        **options,
+    ) -> TaskResult:
+        """
+        Enqueue a task asynchronously.
+
+        Args:
+            task_name: Name of the registered task to execute.
+            args: Positional arguments to pass to the task function.
+            kwargs: Keyword arguments to pass to the task function.
+            **options: Additional adapter-specific options for task execution.
+
+        Returns:
+            TaskResult: Object containing task ID, status, and other metadata.
+
+        Raises:
+            NotImplementedError: If subclass didn't implement enqueue_task_async function
+        """
+
+        raise NotImplementedError("Subclass must implement enqueue_task_async function")
+
+    async def get_task_status_async(self, task_id: str) -> TaskResult:
+        """
+        Get task status asynchronously.
+
+        Args:
+            task_id: Unique identifier of the task to query.
+
+        Returns:
+            TaskResult: Object containing current task status, result, and other metadata.
+
+        Raises:
+            NotImplementedError: If subclass didn't implement get_task_status_async function
+        """
+
+        raise NotImplementedError(
+            "Subclass must implement get_task_status_async function"
+        )
+
+    async def cancel_task_async(self, task_id: str):
+        """
+        Cancel a task.
+
+        Args:
+            task_id: Unique identifier of the task to cancel.
+
+        Raises:
+            NotImplementedError: If subclass didn't implement cancel_task_async function
+        """
+
+        raise NotImplementedError("Subclass must implement cancel_task_async function")
+
+    async def get_metrics_async(self) -> Dict[str, Any]:
+        """
+        Get metrics asynchronously.
+
+        Returns:
+            Dict[str, Any]: Adapter-specific metrics data.
+
+        Raises:
+            NotImplementedError: If subclass didn't implement get_metrics_async function
+        """
+
+        raise NotImplementedError("Subclass must implement get_metrics_async function")
+
+    async def health_check_async(self) -> List[Dict]:
+        """
+        Perform health check asynchronously.
+
+        Returns:
+            List[Dict]: Health status information for workers/components.
+
+        Raises:
+            NotImplementedError: If subclass didn't implement health_check_async function
+        """
+
+        raise NotImplementedError("Subclass must implement health_check_async function")
+
+
 @PublicAPI(stability="alpha")
 class ScaleDeploymentRequest(BaseModel):
     """Request schema for scaling a deployment's replicas."""
@@ -1368,3 +1591,18 @@ class ScaleDeploymentRequest(BaseModel):
     target_num_replicas: NonNegativeInt = Field(
         description="The target number of replicas for the deployment."
     )
+
+
+@PublicAPI(stability="alpha")
+class ReplicaRank(BaseModel):
+    """Replica rank model."""
+
+    rank: int = Field(
+        description="Global rank of the replica across all nodes scoped to the deployment."
+    )
+
+    node_rank: int = Field(description="Rank of the node in the deployment.")
+
+    local_rank: int = Field(
+        description="Rank of the replica on the node scoped to the deployment."
+    )
