@@ -28,16 +28,11 @@ class ThreadRunner:
         self._lock = threading.Lock()
         self._exc_queue: queue.SimpleQueue[Optional[Exception]] = queue.SimpleQueue()
 
-        self._is_running = False
-
     def run(self, target: Callable[[], T]) -> None:
         if self._thread is not None:
             raise RuntimeError("Thread is already running.")
 
         def _run_target():
-            with self._lock:
-                self._is_running = True
-
             try:
                 result = target()
                 with self._lock:
@@ -51,8 +46,16 @@ class ThreadRunner:
                     construct_user_exception_with_traceback(e, exclude_frames=3)
                 )
 
-            with self._lock:
-                self._is_running = False
+            # Join the monitor thread. This ensures that a queued exception
+            # is processed before the target function is considered done.
+            self._monitor_thread.join()
+
+        self._monitor_thread = threading.Thread(
+            target=self._monitor_target,
+            daemon=True,
+            name=f"MonitoringThread({get_callable_name(target)})",
+        )
+        self._monitor_thread.start()
 
         self._thread = threading.Thread(
             target=_run_target,
@@ -61,22 +64,21 @@ class ThreadRunner:
         )
         self._thread.start()
 
-        def _monitor_target():
-            exc = self._exc_queue.get()
-            with self._lock:
-                self._exc = exc
+    def _monitor_target(self):
+        """Monitor the exception queue and set the exception if an exception is found.
 
-        self._monitor_thread = threading.Thread(
-            target=_monitor_target,
-            daemon=True,
-            name=f"MonitoringThread({get_callable_name(target)})",
-        )
-        self._monitor_thread.start()
+        This should run as a daemon thread and exit when None is put into the exception queue.
+        """
+        exc: Optional[UserExceptionWithTraceback] = self._exc_queue.get()
+        if exc is None:
+            return
+
+        with self._lock:
+            self._exc = exc
 
     def is_running(self) -> bool:
         """Returns whether the target function is still running."""
-        with self._lock:
-            return self._is_running
+        return self._thread is not None and self._thread.is_alive()
 
     def get_error(self) -> Optional[BaseException]:
         with self._lock:
