@@ -2,6 +2,7 @@ from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional
 
 from ray.train.v2._internal.data_integration.interfaces import DatasetShardMetadata
 from ray.train.v2._internal.execution.train_fn_utils import get_train_fn_utils
+from ray.train.v2._internal.util import requires_train_worker
 from ray.train.v2.api.context import TrainContext
 from ray.train.v2.api.report_config import CheckpointUploadMode
 from ray.util.annotations import PublicAPI
@@ -13,6 +14,7 @@ if TYPE_CHECKING:
 
 
 @PublicAPI(stability="stable")
+@requires_train_worker(raise_in_tune_session=True)
 def report(
     metrics: Dict[str, Any],
     checkpoint: Optional["Checkpoint"] = None,
@@ -101,16 +103,6 @@ def report(
         validate_config: Configuration passed to the validate_fn. Can contain info
             like the validation dataset.
     """
-    from ray.tune.trainable.trainable_fn_utils import _in_tune_session
-
-    if _in_tune_session():
-        raise DeprecationWarning(
-            "`ray.train.report` is deprecated when running in a function "
-            "passed to Ray Tune. Please use `ray.tune.report` instead. "
-            "See this issue for more context: "
-            "https://github.com/ray-project/ray/issues/49454"
-        )
-
     if delete_local_checkpoint_after_upload is None:
         delete_local_checkpoint_after_upload = (
             checkpoint_upload_mode._default_delete_local_checkpoint_after_upload()
@@ -133,6 +125,7 @@ def report(
 
 
 @PublicAPI(stability="stable")
+@requires_train_worker(raise_in_tune_session=True)
 def get_context() -> TrainContext:
     """Get or create a singleton training context.
 
@@ -140,20 +133,11 @@ def get_context() -> TrainContext:
 
     See the :class:`~ray.train.TrainContext` API reference to see available methods.
     """
-    from ray.tune.trainable.trainable_fn_utils import _in_tune_session
-
-    if _in_tune_session():
-        raise DeprecationWarning(
-            "`ray.train.get_context` is deprecated when running in a function "
-            "passed to Ray Tune. Please use `ray.tune.get_context` instead. "
-            "See this issue for more context: "
-            "https://github.com/ray-project/ray/issues/49454"
-        )
-
     return get_train_fn_utils().get_context()
 
 
 @PublicAPI(stability="stable")
+@requires_train_worker(raise_in_tune_session=True)
 def get_checkpoint() -> Optional["Checkpoint"]:
     """Access the latest reported checkpoint to resume from if one exists.
 
@@ -197,20 +181,11 @@ def get_checkpoint() -> Optional["Checkpoint"]:
         Checkpoint object if the session is currently being resumed.
             Otherwise, return None.
     """
-    from ray.tune.trainable.trainable_fn_utils import _in_tune_session
-
-    if _in_tune_session():
-        raise DeprecationWarning(
-            "`ray.train.get_checkpoint` is deprecated when running in a function "
-            "passed to Ray Tune. Please use `ray.tune.get_checkpoint` instead. "
-            "See this issue for more context: "
-            "https://github.com/ray-project/ray/issues/49454"
-        )
-
     return get_train_fn_utils().get_checkpoint()
 
 
 @PublicAPI(stability="alpha")
+@requires_train_worker()
 def get_all_reported_checkpoints() -> List["ReportedCheckpoint"]:
     """Get all the reported checkpoints so far.
 
@@ -256,6 +231,7 @@ def get_all_reported_checkpoints() -> List["ReportedCheckpoint"]:
 
 
 @PublicAPI(stability="stable")
+@requires_train_worker()
 def get_dataset_shard(dataset_name: Optional[str] = None) -> Optional["DataIterator"]:
     """Returns the :class:`ray.data.DataIterator` shard for this worker.
 
