@@ -18,8 +18,13 @@ import ray
 import ray.core.generated.ray_client_pb2 as ray_client_pb2
 import ray.core.generated.ray_client_pb2_grpc as ray_client_pb2_grpc
 import ray.core.generated.runtime_env_agent_pb2 as runtime_env_agent_pb2
-from ray._common.network_utils import build_address, is_localhost
+from ray._common.network_utils import build_address, is_ipv6, is_localhost
+from ray._private.authentication.http_token_authentication import (
+    format_authentication_http_error,
+    get_auth_headers_if_auth_enabled,
+)
 from ray._private.client_mode_hook import disable_client_hook
+from ray._private.grpc_utils import init_grpc_channel
 from ray._private.parameter import RayParams
 from ray._private.runtime_env.context import RuntimeEnvContext
 from ray._private.services import ProcessInfo, start_ray_client_server
@@ -27,6 +32,7 @@ from ray._private.tls_utils import add_port_to_grpc_server
 from ray._private.utils import detect_fate_sharing_support
 from ray._raylet import GcsClient
 from ray.cloudpickle.compat import pickle
+from ray.exceptions import AuthenticationError
 from ray.job_config import JobConfig
 from ray.util.client.common import (
     CLIENT_SERVER_MAX_THREADS,
@@ -136,7 +142,7 @@ class ProxyManager:
         self._node: Optional[ray._private.node.Node] = None
         atexit.register(self._cleanup)
 
-    def _get_unused_port(self) -> int:
+    def _get_unused_port(self, family: int = socket.AF_INET) -> int:
         """
         Search for a port in _free_ports that is unused.
         """
@@ -144,7 +150,7 @@ class ProxyManager:
             num_ports = len(self._free_ports)
             for _ in range(num_ports):
                 port = self._free_ports.pop(0)
-                s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+                s = socket.socket(family, socket.SOCK_STREAM)
                 try:
                     s.bind(("", port))
                 except OSError:
@@ -197,12 +203,17 @@ class ProxyManager:
             assert (
                 self.servers.get(client_id) is None
             ), f"Server already created for Client: {client_id}"
-            port = self._get_unused_port()
+
+            host = "127.0.0.1"
+            port = self._get_unused_port(
+                socket.AF_INET6 if is_ipv6(host) else socket.AF_INET
+            )
+
             server = SpecificServer(
                 port=port,
                 process_handle_future=futures.Future(),
-                channel=ray._private.utils.init_grpc_channel(
-                    build_address("127.0.0.1", port), options=GRPC_OPTIONS
+                channel=init_grpc_channel(
+                    build_address(host, port), options=GRPC_OPTIONS
                 ),
             )
             self.servers[client_id] = server
@@ -246,8 +257,11 @@ class ProxyManager:
                     self._runtime_env_agent_address, "/get_or_create_runtime_env"
                 )
                 data = create_env_request.SerializeToString()
-                req = urllib.request.Request(url, data=data, method="POST")
-                req.add_header("Content-Type", "application/octet-stream")
+                headers = {"Content-Type": "application/octet-stream"}
+                headers.update(**get_auth_headers_if_auth_enabled(headers))
+                req = urllib.request.Request(
+                    url, data=data, method="POST", headers=headers
+                )
                 response = urllib.request.urlopen(req, timeout=None)
                 response_data = response.read()
                 r = runtime_env_agent_pb2.GetOrCreateRuntimeEnvReply()
@@ -265,6 +279,25 @@ class ProxyManager:
                     )
                 else:
                     assert False, f"Unknown status: {r.status}."
+            except urllib.error.HTTPError as e:
+                body = ""
+                try:
+                    body = e.read().decode("utf-8", "ignore")
+                except Exception:
+                    body = e.reason if hasattr(e, "reason") else str(e)
+
+                formatted_error = format_authentication_http_error(e.code, body or "")
+                if formatted_error:
+                    raise AuthenticationError(formatted_error) from e
+
+                # Treat non-auth HTTP errors like URLError (retry with backoff)
+                last_exception = e
+                logger.warning(
+                    f"GetOrCreateRuntimeEnv request failed with HTTP {e.code}: {body or e}. "
+                    f"Retrying after {wait_time_s}s. "
+                    f"{max_retries-retries} retries remaining."
+                )
+
             except urllib.error.URLError as e:
                 last_exception = e
                 logger.warning(
@@ -843,9 +876,13 @@ def serve_proxier(
         gcs_cli = GcsClient(address=gcs_address)
         ray.experimental.internal_kv._initialize_internal_kv(gcs_cli)
 
-    server = grpc.server(
-        futures.ThreadPoolExecutor(max_workers=CLIENT_SERVER_MAX_THREADS),
+    from ray._private.grpc_utils import create_grpc_server_with_interceptors
+
+    server = create_grpc_server_with_interceptors(
+        max_workers=CLIENT_SERVER_MAX_THREADS,
+        thread_name_prefix="ray_client_proxier",
         options=GRPC_OPTIONS,
+        asynchronous=False,
     )
     proxy_manager = ProxyManager(
         gcs_address,
