@@ -2,6 +2,7 @@
 It implements the Ray API functions that are forwarded through grpc calls
 to the server.
 """
+
 import base64
 import json
 import logging
@@ -17,11 +18,14 @@ from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Tuple, Un
 
 import grpc
 
-import ray._private.tls_utils
 import ray.cloudpickle as cloudpickle
 import ray.core.generated.ray_client_pb2 as ray_client_pb2
 import ray.core.generated.ray_client_pb2_grpc as ray_client_pb2_grpc
-from ray._private.ray_constants import DEFAULT_CLIENT_RECONNECT_GRACE_PERIOD
+from ray._private.ray_constants import (
+    DEFAULT_CLIENT_RECONNECT_GRACE_PERIOD,
+    env_float,
+    env_integer,
+)
 from ray._private.runtime_env.py_modules import upload_py_modules_if_needed
 from ray._private.runtime_env.working_dir import upload_working_dir_if_needed
 
@@ -52,13 +56,14 @@ if TYPE_CHECKING:
 
 logger = logging.getLogger(__name__)
 
-INITIAL_TIMEOUT_SEC = 5
-MAX_TIMEOUT_SEC = 30
-
+INITIAL_TIMEOUT_SEC = env_integer("RAY_CLIENT_INITIAL_CONNECTION_TIMEOUT_S", 5)
+MAX_TIMEOUT_SEC = env_integer("RAY_CLIENT_MAX_CONNECTION_TIMEOUT_S", 30)
 # The max amount of time an operation can run blocking in the server. This
 # allows for Ctrl-C of the client to work without explicitly cancelling server
 # operations.
-MAX_BLOCKING_OPERATION_TIME_S: float = 2.0
+MAX_BLOCKING_OPERATION_TIME_S: float = env_float(
+    "RAY_CLIENT_MAX_BLOCKING_OPERATION_TIME_S", 2.0
+)
 
 # If the total size (bytes) of all outbound messages to schedule tasks since
 # the connection began exceeds this value, a warning should be raised
@@ -168,27 +173,28 @@ class Worker:
             self.channel.unsubscribe(self._on_channel_state_change)
             self.channel.close()
 
+        from ray._private.grpc_utils import init_grpc_channel
+
+        # Prepare credentials if secure connection is requested
+        credentials = None
         if self._secure:
             if self._credentials is not None:
                 credentials = self._credentials
             elif os.environ.get("RAY_USE_TLS", "0").lower() in ("1", "true"):
-                (
-                    server_cert_chain,
-                    private_key,
-                    ca_cert,
-                ) = ray._private.tls_utils.load_certs_from_env()
-                credentials = grpc.ssl_channel_credentials(
-                    certificate_chain=server_cert_chain,
-                    private_key=private_key,
-                    root_certificates=ca_cert,
-                )
+                # init_grpc_channel will handle this via load_certs_from_env()
+                credentials = None
             else:
+                # Default SSL credentials (no specific certs)
                 credentials = grpc.ssl_channel_credentials()
-            self.channel = grpc.secure_channel(
-                self._conn_str, credentials, options=GRPC_OPTIONS
-            )
-        else:
-            self.channel = grpc.insecure_channel(self._conn_str, options=GRPC_OPTIONS)
+
+        # Create channel with auth interceptors via helper
+        # This automatically adds auth interceptors when token auth is enabled
+        self.channel = init_grpc_channel(
+            self._conn_str,
+            options=GRPC_OPTIONS,
+            asynchronous=False,
+            credentials=credentials,
+        )
 
         self.channel.subscribe(self._on_channel_state_change)
 
@@ -228,15 +234,14 @@ class Worker:
                 # which is why we do not sleep here.
             except grpc.RpcError as e:
                 logger.debug(
-                    "Ray client server unavailable, " f"retrying in {timeout}s..."
+                    f"Ray client server unavailable, retrying in {timeout}s..."
                 )
                 logger.debug(f"Received when checking init: {e.details()}")
                 # Ray is not ready yet, wait a timeout.
                 time.sleep(timeout)
             # Fallthrough, backoff, and retry at the top of the loop
             logger.debug(
-                "Waiting for Ray to become ready on the server, "
-                f"retry in {timeout}s..."
+                f"Waiting for Ray to become ready on the server, retry in {timeout}s..."
             )
             if not reconnecting:
                 # Don't increase backoff when trying to reconnect --
@@ -416,19 +421,14 @@ class Worker:
         else:
             deadline = time.monotonic() + timeout
 
-        max_blocking_operation_time = MAX_BLOCKING_OPERATION_TIME_S
-        if "RAY_CLIENT_MAX_BLOCKING_OPERATION_TIME_S" in os.environ:
-            max_blocking_operation_time = float(
-                os.environ["RAY_CLIENT_MAX_BLOCKING_OPERATION_TIME_S"]
-            )
         while True:
             if deadline:
                 op_timeout = min(
-                    max_blocking_operation_time,
+                    MAX_BLOCKING_OPERATION_TIME_S,
                     max(deadline - time.monotonic(), 0.001),
                 )
             else:
-                op_timeout = max_blocking_operation_time
+                op_timeout = MAX_BLOCKING_OPERATION_TIME_S
             try:
                 res = self._get(to_get, op_timeout)
                 break
@@ -523,7 +523,7 @@ class Worker:
     ) -> Tuple[List[ClientObjectRef], List[ClientObjectRef]]:
         if not isinstance(object_refs, list):
             raise TypeError(
-                "wait() expected a list of ClientObjectRef, " f"got {type(object_refs)}"
+                f"wait() expected a list of ClientObjectRef, got {type(object_refs)}"
             )
         for ref in object_refs:
             if not isinstance(ref, ClientObjectRef):
